@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,14 +20,15 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
-open OUnit;;
 
-let myrun t = 
-  let lst = 
+open OUnit
+
+
+let myrun t =
+  let lst =
     run_test_tt_main t
   in
     lst
-;;
 
-myrun ("main" >:: (fun () -> ()))
-;;
+
+let _ = myrun ("main" >:: (fun () -> ()))
