@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,29 +20,36 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 open OASISTypes
 open OASISGettext
 open BaseStandardVar
 open BaseMessage
 
+
 type t =
   | BExec    (* Executable *)
   | BExecLib (* Library coming with executable *)
   | BLib     (* Library *)
+  | BObj     (* Library *)
   | BDoc     (* Document *)
 
+
 let to_log_event_file t nm =
   "built_"^
   (match t with
      | BExec -> "exec"
      | BExecLib -> "exec_lib"
      | BLib -> "lib"
+     | BObj -> "obj"
      | BDoc -> "doc")^
   "_"^nm
 
+
 let to_log_event_done t nm =
   "is_"^(to_log_event_file t nm)
 
+
 let register t nm lst =
   BaseLog.register
     (to_log_event_done t nm)
@@ -72,6 +80,7 @@ let register t nm lst =
              (String.concat (s_ ", ") alt))
     lst
 
+
 let unregister t nm =
   List.iter
     (fun (e, d) ->
@@ -80,6 +89,7 @@ let unregister t nm =
        [to_log_event_file t nm;
         to_log_event_done t nm])
 
+
 let fold t nm f acc =
   List.fold_left
     (fun acc (_, fn) ->
@@ -99,6 +109,8 @@ let fold t nm f acc =
                        (f_ "executable %s")
                    | BLib ->
                        (f_ "library %s")
+                   | BObj ->
+                       (f_ "object %s")
                    | BDoc ->
                        (f_ "documentation %s"))
                 nm);
@@ -108,6 +120,7 @@ let fold t nm f acc =
     (BaseLog.filter
        [to_log_event_file t nm])
 
+
 let is_built t nm =
   List.fold_left
     (fun is_built (_, d) ->
@@ -119,6 +132,7 @@ let is_built t nm =
     (BaseLog.filter
        [to_log_event_done t nm])
 
+
 let of_executable ffn (cs, bs, exec) =
   let unix_exec_is, unix_dll_opt =
     OASISExecutable.unix_exec_is
@@ -142,6 +156,7 @@ let of_executable ffn (cs, bs, exec) =
     unix_exec_is,
     unix_dll_opt
 
+
 let of_library ffn (cs, bs, lib) =
   let unix_lst =
     OASISLibrary.generated_unix_files
@@ -161,3 +176,20 @@ let of_library ffn (cs, bs, lib) =
   in
     evs, unix_lst
 
+
+let of_object ffn (cs, bs, obj) =
+  let unix_lst =
+    OASISObject.generated_unix_files
+      ~ctxt:!BaseContext.default
+      ~source_file_exists:(fun fn ->
+         OASISFileUtil.file_exists_case (OASISHostPath.of_unix fn))
+      ~is_native:(bool_of_string (is_native ()))
+      (cs, bs, obj)
+  in
+  let evs =
+    [BObj,
+     cs.cs_name,
+     List.map (List.map ffn) unix_lst]
+  in
+    evs, unix_lst
+
