@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,11 +20,13 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 open BaseEnv
 open BaseMessage
 open OASISTypes
 open OASISGettext
 
+
 let doc lst pkg extra_args =
 
   let one_doc (doc_plugin, cs, doc) =
@@ -43,7 +46,7 @@ let doc lst pkg extra_args =
   in
     List.iter one_doc lst;
 
-    if OASISVersion.version_0_3_or_after pkg.oasis_version &&
+    if OASISFeatures.package_test OASISFeatures.flag_docs pkg &&
        not (bool_of_string (BaseStandardVar.docs ())) &&
        lst <> [] then
       BaseMessage.warning
