@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -25,6 +26,7 @@ open OASISGettext
 open BaseEnv
 open BaseBuilt
 
+
 let init pkg =
   (* TODO: disambiguate exec vs other variable by adding exec_VARNAME. *)
   (* TODO: provide compile option for library libary_byte_args_VARNAME... *)
@@ -58,6 +60,6 @@ let init pkg =
                                            (f_ "Executable '%s' not yet built.")
                                            cs.cs_name)))))
 
-       | Library _ | Flag _ | Test _ | SrcRepo _ | Doc _ ->
+       | Library _ | Object _ | Flag _ | Test _ | SrcRepo _ | Doc _ ->
            ())
     pkg.sections
