@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -23,14 +24,17 @@ open OASISGettext
 open OASISUtils
 open PropList
 
+
 module MapString = BaseEnvLight.MapString
 
+
 type origin_t =
   | ODefault
   | OGetEnv
   | OFileLoad
   | OCommandLine
 
+
 type cli_handle_t =
   | CLINone
   | CLIAuto
@@ -38,6 +42,7 @@ type cli_handle_t =
   | CLIEnable
   | CLIUser of (Arg.key * Arg.spec * Arg.doc) list
 
+
 type definition_t =
     {
       hide:       bool;
@@ -47,21 +52,26 @@ type definition_t =
       group:      string option;
     }
 
+
 let schema =
   Schema.create "environment"
 
+
 (* Environment data *)
 let env =
   Data.create ()
 
+
 (* Environment data from file *)
 let env_from_file =
   ref MapString.empty
 
+
 (* Lexer for var *)
 let var_lxr =
   Genlex.make_lexer []
 
+
 let rec var_expand str =
   let buff =
     Buffer.create ((String.length str) * 2)
@@ -112,6 +122,7 @@ let rec var_expand str =
       str;
     Buffer.contents buff
 
+
 and var_get name =
   let vl =
     try
@@ -126,6 +137,7 @@ and var_get name =
   in
     var_expand vl
 
+
 let var_choose ?printer ?name lst =
   OASISExpr.choose
     ?printer
@@ -133,6 +145,7 @@ let var_choose ?printer ?name lst =
     var_get
     lst
 
+
 let var_protect vl =
   let buff =
     Buffer.create (String.length vl)
@@ -144,6 +157,7 @@ let var_protect vl =
       vl;
     Buffer.contents buff
 
+
 let var_define
       ?(hide=false)
       ?(dump=true)
@@ -229,6 +243,7 @@ let var_define
     fun () ->
       var_expand (var_get_low (var_get_lst env))
 
+
 let var_redefine
       ?hide
       ?dump
@@ -257,8 +272,9 @@ let var_redefine
         dflt
     end
 
-let var_ignore (e : unit -> string) =
-  ()
+
+let var_ignore (e: unit -> string) = ()
+
 
 let print_hidden =
   var_define
@@ -269,6 +285,7 @@ let print_hidden =
     "print_hidden"
     (fun () -> "false")
 
+
 let var_all () =
   List.rev
     (Schema.fold
@@ -280,24 +297,28 @@ let var_all () =
        []
        schema)
 
+
 let default_filename =
   BaseEnvLight.default_filename
 
+
 let load ?allow_empty ?filename () =
   env_from_file := BaseEnvLight.load ?allow_empty ?filename ()
 
+
 let unload () =
   env_from_file := MapString.empty;
   Data.clear env
 
+
 let dump ?(filename=default_filename) () =
   let chn =
     open_out_bin filename
   in
-  let output nm value = 
+  let output nm value =
     Printf.fprintf chn "%s=%S\n" nm value
   in
-  let mp_todo = 
+  let mp_todo =
     (* Dump data from schema *)
     Schema.fold
       (fun mp_todo nm def _ ->
@@ -324,6 +345,7 @@ let dump ?(filename=default_filename) () =
     (* End of the dump *)
     close_out chn
 
+
 let print () =
   let printable_vars =
     Schema.fold
@@ -362,11 +384,12 @@ let print () =
 
   Printf.printf "\nConfiguration: \n";
   List.iter
-    (fun (name,value) ->
+    (fun (name, value) ->
       Printf.printf "%s: %s %s\n" name (dot_pad name) value)
     (List.rev printable_vars);
   Printf.printf "\n%!"
 
+
 let args () =
   let arg_concat =
     OASISUtils.varname_concat ~hyphen:'-'
