@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,15 +20,19 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 module MapString = Map.Make(String)
 
+
 type t = string MapString.t
 
+
 let default_filename =
   Filename.concat
     (Sys.getcwd ())
     "setup.data"
 
+
 let load ?(allow_empty=false) ?(filename=default_filename) () =
   if Sys.file_exists filename then
     begin
@@ -85,26 +90,29 @@ let load ?(allow_empty=false) ?(filename=default_filename) () =
            filename)
     end
 
-let var_get name env =
-  let rec var_expand str =
-    let buff =
-      Buffer.create ((String.length str) * 2)
-    in
-      Buffer.add_substitute
-        buff
-        (fun var ->
-           try
-             var_expand (MapString.find var env)
-           with Not_found ->
-             failwith
-               (Printf.sprintf
-                  "No variable %s defined when trying to expand %S."
-                  var
-                  str))
-        str;
-      Buffer.contents buff
+
+let rec var_expand str env =
+  let buff =
+    Buffer.create ((String.length str) * 2)
   in
-    var_expand (MapString.find name env)
+    Buffer.add_substitute
+      buff
+      (fun var ->
+         try
+           var_expand (MapString.find var env) env
+         with Not_found ->
+           failwith
+             (Printf.sprintf
+                "No variable %s defined when trying to expand %S."
+                var
+                str))
+      str;
+    Buffer.contents buff
+
+
+let var_get name env =
+  var_expand (MapString.find name env) env
+
 
 let var_choose lst env =
   OASISExpr.choose
