@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -26,12 +27,15 @@ open OASISSection
 open OASISGettext
 open OASISUtils
 
+
 type std_args_fun =
     package -> string array -> unit
 
+
 type ('a, 'b) section_args_fun =
     name * (package -> (common_section * 'a) -> string array -> 'b)
 
+
 type t =
     {
       configure:        std_args_fun;
@@ -55,6 +59,7 @@ type t =
       setup_update:     bool;
     }
 
+
 (* Associate a plugin function with data from package *)
 let join_plugin_sections filter_map lst =
   List.rev
@@ -68,6 +73,7 @@ let join_plugin_sections filter_map lst =
        []
        lst)
 
+
 (* Search for plugin data associated with a section name *)
 let lookup_plugin_section plugin action nm lst =
   try
@@ -79,11 +85,12 @@ let lookup_plugin_section plugin action nm lst =
       nm
       action
 
+
 let configure t args =
   (* Run configure *)
   BaseCustom.hook
     t.package.conf_custom
-    (fun () -> 
+    (fun () ->
        (* Reload if preconf has changed it *)
        begin
          try
@@ -110,12 +117,14 @@ let configure t args =
   (* Replace data in file *)
   BaseFileAB.replace t.package.files_ab
 
+
 let build t args =
   BaseCustom.hook
     t.package.build_custom
     (t.build t.package)
     args
 
+
 let doc t args =
   BaseDoc.doc
     (join_plugin_sections
@@ -135,6 +144,7 @@ let doc t args =
     t.package
     args
 
+
 let test t args =
   BaseTest.test
     (join_plugin_sections
@@ -154,12 +164,16 @@ let test t args =
     t.package
     args
 
+
 let all t args =
   let rno_doc =
     ref false
   in
   let rno_test =
     ref false
+  in
+  let arg_rest =
+    ref []
   in
     Arg.parse_argv
       ~current:(ref 0)
@@ -174,12 +188,16 @@ let all t args =
         "-no-test",
         Arg.Set rno_test,
         s_ "Don't run test target";
+
+        "--",
+        Arg.Rest (fun arg -> arg_rest := arg :: !arg_rest),
+        s_ "All arguments for configure.";
       ]
       (failwithf (f_ "Don't know what to do with '%s'"))
       "";
 
     info "Running configure step";
-    configure t [||];
+    configure t (Array.of_list (List.rev !arg_rest));
 
     info "Running build step";
     build     t [||];
@@ -207,22 +225,26 @@ let all t args =
         info "Skipping test step"
       end
 
+
 let install t args =
   BaseCustom.hook
     t.package.install_custom
     (t.install t.package)
     args
 
+
 let uninstall t args =
   BaseCustom.hook
     t.package.uninstall_custom
     (t.uninstall t.package)
     args
 
+
 let reinstall t args =
   uninstall t args;
   install t args
 
+
 let clean, distclean =
   let failsafe f a =
     try
@@ -264,6 +286,7 @@ let clean, distclean =
                       (f t.package (cs, doc))
                       args
               | Library _
+              | Object _
               | Executable _
               | Flag _
               | SrcRepo _ ->
@@ -319,9 +342,11 @@ let clean, distclean =
 
     clean, distclean
 
+
 let version t _ =
   print_endline t.oasis_version
 
+
 let update_setup_ml, no_update_setup_ml_cli =
   let b = ref true in
     b,
@@ -329,11 +354,15 @@ let update_setup_ml, no_update_setup_ml_cli =
      Arg.Clear b,
      s_ " Don't try to update setup.ml, even if _oasis has changed.")
 
+
+let default_oasis_fn = "_oasis"
+
+
 let update_setup_ml t =
   let oasis_fn =
     match t.oasis_fn with
       | Some fn -> fn
-      | None -> "_oasis"
+      | None -> default_oasis_fn
   in
   let oasis_exec =
     match t.oasis_exec with
@@ -431,7 +460,8 @@ let update_setup_ml t =
       try
         match t.oasis_digest with
           | Some dgst ->
-            if Sys.file_exists oasis_fn && dgst <> Digest.file "_oasis" then
+            if Sys.file_exists oasis_fn &&
+               dgst <> Digest.file default_oasis_fn then
               begin
                 do_update ();
                 true
@@ -451,6 +481,7 @@ let update_setup_ml t =
   else
     false
 
+
 let setup t =
   let catch_exn =
     ref true
@@ -592,12 +623,15 @@ let setup t =
       error "%s" (Printexc.to_string e);
       exit 1
 
+
 (* END EXPORT *)
 
+
 open OASISPlugin
 
-let default_filename =
-  "setup.ml"
+
+let default_filename = "setup.ml"
+
 
 let find ctxt =
   try
@@ -608,15 +642,18 @@ let find ctxt =
       default_filename
 
 
-let of_package ?oasis_fn ?oasis_exec ?(oasis_setup_args=[]) ~setup_update pkg =
+let of_package ?oasis_fn ?oasis_exec ?(oasis_setup_args=[]) ~setup_update update pkg =
 
   let ctxt =
     (* Initial context *)
     {
       error         = false;
-      files         = OASISFileTemplate.empty;
       other_actions = [];
       ctxt          = !BaseContext.default;
+      update        = update;
+      files = OASISFileTemplate.create
+                ~disable_oasis_section:pkg.disable_oasis_section
+                ();
     }
   in
 
@@ -892,6 +929,7 @@ let of_package ?oasis_fn ?oasis_exec ?(oasis_setup_args=[]) ~setup_update pkg =
      ["let () = setup ();;"]
   in
 
+
     {ctxt with
          files = OASISFileTemplate.replace setup_tmpl ctxt.files},
     t
