@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,12 +20,14 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 open BaseEnv
 open BaseMessage
 open OASISTypes
 open OASISExpr
 open OASISGettext
 
+
 let test lst pkg extra_args =
 
   let one_test (failure, n) (test_plugin, cs, test) =
@@ -75,7 +78,7 @@ let test lst pkg extra_args =
         (failure, n)
       end
   in
-  let (failed, n) =
+  let failed, n =
     List.fold_left
       one_test
       (0.0, 0)
@@ -98,7 +101,7 @@ let test lst pkg extra_args =
       info "%s" msg;
 
     (* Possible explanation why the tests where not run. *)
-    if OASISVersion.version_0_3_or_after pkg.oasis_version &&
+    if OASISFeatures.package_test OASISFeatures.flag_tests pkg &&
        not (bool_of_string (BaseStandardVar.tests ())) &&
        lst <> [] then
       BaseMessage.warning
