@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,34 +20,26 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (** Check an _oasis file
     @author Sylvain Le Gall
   *)
 
+
 open OASISGettext
-open SubCommand
+open CLISubCommand
 
-let main () = 
-  let _pkg: OASISTypes.package = 
-    OASISParse.from_file
-      ~ctxt:{!BaseContext.default with 
-                 OASISContext.ignore_plugins = !ArgCommon.ignore_plugins}
-      !ArgCommon.oasis_fn
-  in
-    ()
 
-let scmd =
-  {(SubCommand.make 
-      ~std_usage:true
-      "check"
-      (s_ "Check an _oasis file")
-      CLIData.check_mkd
-      main) 
-     with 
-         scmd_specs = 
-           (ArgCommon.ignore_plugins_specs 
-            @
-            ArgCommon.oasis_fn_specs)} 
+let main ~ctxt () oasis_fn pkg =
+  (* Side effect of parsing pkg, it is already checked. *)
+  ()
 
-let () = 
-  SubCommand.register scmd
+
+let () =
+  CLISubCommand.register "check"
+    (ns_ "Check an _oasis file")
+    CLIData.check_mkd
+    (CLICommon.parse_oasis_fn
+       (CLISubCommand.make_run
+          CLISubCommand.default_fspecs
+          main))
