@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,46 +20,43 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
-(** Help on subcommands 
+
+(** Help on subcommands
   *)
 
-open SubCommand 
-open ArgExt
+
+open CLISubCommand
+open CLIArgExt
 open OASISGettext
 
-let scmd_name = 
-  ref None
 
-let main () = 
-  let pp_print_help =
-    match !scmd_name with 
-      | None ->
-          pp_print_help NoSubCommand
-      | Some "all" ->
-          pp_print_help AllSubCommand
-      | Some nm ->
-          pp_print_help (SubCommand nm)
+let main ~ctxt scmd_name =
+  let hext =
+    match scmd_name with
+      | None -> NoSubCommand
+      | Some "all" -> AllSubCommand
+      | Some nm -> (SubCommand nm)
   in
-  let pager, fmt = 
-    Pager.open_out ()
+  let pager, fmt =
+    CLIPager.open_out ()
   in
     try
-      pp_print_help Output fmt ();
-      Pager.close_out pager
+      pp_print_help ~ctxt hext Output fmt ();
+      CLIPager.close_out pager
     with e ->
-      Pager.close_out pager;
+      CLIPager.close_out pager;
       raise e
 
-let scmd = 
-  {(SubCommand.make
-      "help"
-      (s_ "Display help for a subcommand")
-      CLIData.help_mkd
-      main)
-     with 
-         scmd_usage = s_ "[subcommand|all]";
-         scmd_anon  = (fun s -> scmd_name := Some s)}
 
-let () = 
-  SubCommand.register scmd
-    
+let () =
+  CLISubCommand.register "help"
+    (ns_ "Display help for a subcommand")
+    CLIData.help_mkd
+    ~usage:(ns_ "[subcommand|all]")
+    (CLISubCommand.make_run
+       (fun () ->
+          let scmd_name = ref None in
+            ([], (fun s -> scmd_name := Some s)),
+            (fun () -> !scmd_name))
+       main)
+
