@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,16 +20,19 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (** Main for OASIS *)
 
+
 open MainGettext
 
+
 open OASISTypes
 open OASISUtils
 open OASISPlugin
 open OASISBuiltinPlugins
 open BaseMessage
-open SubCommand
+open CLISubCommand
 open Format
 open FormatExt
 
@@ -41,24 +45,14 @@ open Manual
 open Check
 open Query
 open Version
-open PluginDB
-open PluginInstall
-open PluginVCS
-open PluginBundle
-open Plugins
 open Help
 
 
 let () =
   (* Run subcommand *)
-  try 
-    let () = 
-      OASISBuiltinPlugins.init ()
-    in
-    let main =
-      ArgExt.parse ()
-    in
-      main ()
+  try
+    OASISBuiltinPlugins.init ();
+    CLIArgExt.parse_and_run ()
   with e ->
     begin
       if Printexc.backtrace_status () then
@@ -69,7 +63,13 @@ let () =
           | Failure str ->
               error "%s" str
           | e ->
-              error "%s" (Printexc.to_string e)
+              begin
+                error "%s" (Printexc.to_string e);
+                if Printexc.backtrace_status () then
+                  List.iter
+                    (debug "%s")
+                    (OASISString.nsplit (Printexc.get_backtrace ()) '\n')
+              end
       end;
 
       exit 1
