@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,45 +20,46 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (** Display the manual
     @author Sylvain Le Gall
   *)
 
-open SubCommand
+
+open CLISubCommand
 open OASISGettext
+open CLIArgExt
 
-let output = 
-  ref None
 
-let main () = 
-  let fmt, fclose = 
-    match !output with 
+let main ~ctxt output =
+  let fmt, fclose =
+    match output with
       | Some fn ->
-          let chn = 
+          let chn =
             open_out fn
           in
             Format.formatter_of_out_channel chn,
             (fun () -> close_out chn)
 
       | None ->
-          let pager, fmt = 
-            Pager.open_out ()
+          let pager, fmt =
+            CLIPager.open_out ()
           in
             fmt,
-            (fun () -> Pager.close_out pager)
+            (fun () -> CLIPager.close_out pager)
   in
-    
-    OASISHelp.pp_print_help 
+
+    OASISHelp.pp_print_help
       fmt
 
       (* CLI help *)
-      (ArgExt.pp_print_help ArgExt.AllSubCommand ArgExt.Markdown)
+      (pp_print_help ~ctxt AllSubCommand Markdown)
 
       (* Fields from schema *)
       BaseEnv.schema
 
       (* Environment variable *)
-      (let lst = 
+      (let lst =
          BaseEnv.var_all ()
        in
          fun nm _ ->
@@ -65,16 +67,17 @@ let main () =
 
     fclose ()
 
-let scmd = 
-  {(SubCommand.make
-      "manual"
-      (s_ "Display user manual")
-      CLIData.manual_mkd
-      main)
-     with 
-         scmd_specs = ["-o", 
-                       Arg.String (fun s -> output := Some s),
-                       "fn Output manual to filename."]}
 
-let () = 
-  SubCommand.register scmd
+let () =
+  CLISubCommand.register "manual"
+    (ns_ "Display user manual")
+    CLIData.manual_mkd
+    (CLISubCommand.make_run
+       (fun () ->
+          let output = ref None in
+            (["-o",
+              Arg.String (fun s -> output := Some s),
+              "fn Output manual to filename."],
+             CLISubCommand.default_anon),
+            (fun () -> !output))
+       main)
