@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,11 +20,13 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (** Load and query _oasis file
     @author Sylvain Le Gall
   *)
 
-open SubCommand
+
+open CLISubCommand
 open BaseMessage
 open Genlex
 open OASISGettext
@@ -31,23 +34,19 @@ open OASISTypes
 open OASISSection
 open OASISUtils
 
-let queries =
-  ref []
 
-let separator = 
-  ref "\n"
-
-let lexer = 
-  make_lexer 
+let lexer =
+  make_lexer
     ["ListSections"; "ListFields"; "("; ")"; "."]
 
-let query pkg str = 
-  let proplist_schema schm = 
+
+let query pkg separator str =
+  let proplist_schema schm =
     (* TODO: oops access to unpublished module _intern *)
     schm.OASISSchema_intern.schm
   in
 
-  let assoc_sections = 
+  let assoc_sections =
     [
       "Library", (`Library, proplist_schema OASISLibrary.schema);
 
@@ -63,19 +62,19 @@ let query pkg str =
     ]
   in
 
-  let mk_section sct = 
-    let (knd, nm) =
+  let mk_section sct =
+    let knd, nm =
       OASISSection.section_id sct
     in
-    let start, (_, schm) = 
-      List.find 
+    let start, (_, schm) =
+      List.find
         (fun (str, (knd', _)) -> knd = knd')
         assoc_sections
     in
-    let fmt = 
-      if OASISUtils.is_varname nm then  
-        Printf.sprintf "%s(%s)" 
-      else 
+    let fmt =
+      if OASISUtils.is_varname nm then
+        Printf.sprintf "%s(%s)"
+      else
         Printf.sprintf "%s(%S)"
     in
       fmt start nm, schm
@@ -89,18 +88,18 @@ let query pkg str =
           str
   in
 
-  let parse_fld_or_section start_nm = 
-    parser 
-      | [< 'Kwd "("; nm = parse_id_or_string; 'Kwd ")"; 
-           'Kwd "."; 
-           fld = parse_id_or_string >] ->          
+  let parse_fld_or_section start_nm =
+    parser
+      | [< 'Kwd "("; nm = parse_id_or_string; 'Kwd ")";
+           'Kwd ".";
+           fld = parse_id_or_string >] ->
           begin
-            let kind, schm = 
-              try 
-                List.assoc 
+            let kind, schm =
+              try
+                List.assoc
                   (String.lowercase start_nm)
-                  (List.map 
-                     (fun (nm, e) -> String.lowercase nm, e) 
+                  (List.map
+                     (fun (nm, e) -> String.lowercase nm, e)
                      assoc_sections)
               with Not_found ->
                 failwithf
@@ -109,10 +108,10 @@ let query pkg str =
             in
             let sct  =
               OASISSection.section_find
-                (kind, nm) 
+                (kind, nm)
                 pkg.sections
             in
-            let data = 
+            let data =
               (OASISSection.section_common sct).cs_data
             in
               schm, data, fld
@@ -127,9 +126,9 @@ let query pkg str =
           end
   in
 
-  let parse = 
-    parser 
-      | [< start_nm = parse_id_or_string; 
+  let parse =
+    parser
+      | [< start_nm = parse_id_or_string;
            (schm, data, fld) = parse_fld_or_section start_nm >] ->
           begin
             PropList.Schema.get schm data fld
@@ -137,9 +136,9 @@ let query pkg str =
 
       | [< 'Kwd "ListSections" >] ->
           begin
-            String.concat 
-              !separator
-              (List.map 
+            String.concat
+              separator
+              (List.map
                  (fun sct -> fst (mk_section sct))
                  pkg.sections)
           end
@@ -148,18 +147,24 @@ let query pkg str =
           begin
             let fold_schm prefix schm data acc =
               PropList.Schema.fold
-                (fun acc nm _ _ ->
-                   try 
-                     let _v : string = 
+                (fun acc nm extra _ ->
+                   try
+                     let _v: string =
                        PropList.Schema.get schm data nm
                      in
-                       (prefix^nm) :: acc
+                       match extra.OASISSchema_intern.feature with
+                         | Some ftr ->
+                            if OASISFeatures.package_test ftr pkg then
+                              (prefix^nm) :: acc
+                            else
+                              acc
+                         | None -> (prefix^nm) :: acc
                    with e ->
                      acc)
                 acc
                 schm
             in
-            let lst = 
+            let lst =
               List.fold_left
                 (fun acc sct ->
                    let prefix, schm = mk_section sct in
@@ -167,53 +172,41 @@ let query pkg str =
                      fold_schm (prefix^".") schm data acc)
 
                 (* Start with the package fields *)
-                (fold_schm "" (proplist_schema OASISPackage.schema) pkg.schema_data [])
+                (fold_schm "" (proplist_schema OASISPackage.schema)
+                   pkg.schema_data [])
 
                 (* Continue with section fields *)
                 pkg.sections
             in
 
-              String.concat 
-                !separator 
-                (List.rev lst)
+              String.concat separator (List.rev lst)
           end
 
   in
     parse (lexer (Stream.of_string str))
 
 
-let main () = 
-  let pkg = 
-    OASISParse.from_file
-      ~ctxt:{!BaseContext.default with 
-                 OASISContext.ignore_plugins = !ArgCommon.ignore_plugins}
-      !ArgCommon.oasis_fn
+let main ~ctxt (queries, separator) _ pkg =
+  let answers =
+    List.rev_map (query pkg separator) queries
   in
-  let answers = 
-    List.rev_map (query pkg) !queries
-  in
-    print_endline (String.concat !separator answers)
+    print_endline (String.concat separator answers)
 
 
-let scmd =
-  {(SubCommand.make 
-      ~std_usage:false
-      "query"
-      (s_ "Query an _oasis file")
-      CLIData.query_mkd
-      main) 
-     with
-         scmd_usage = 
-           s_ "[options*] query*";
-         scmd_anon = 
-           (fun e -> queries := e :: !queries);
-         scmd_specs =
-           ([
-             "-separator",
-             Arg.Set_string separator,
-             s_ "str String to add between answers."
-           ]
-           @ ArgCommon.oasis_fn_specs @ ArgCommon.ignore_plugins_specs)}
+let () =
+  CLISubCommand.register "query"
+    ~usage:(ns_ "[options*] query*")
+    (ns_ "Query an _oasis file")
+    CLIData.query_mkd
+    (CLICommon.parse_oasis_fn
+       (CLISubCommand.make_run
+          (fun () ->
+             let separator = ref "\n" in
+             let queries = ref [] in
+               (["-separator",
+                 Arg.Set_string separator,
+                 s_ "str String to add between answers."],
+                (fun e -> queries := e :: !queries)),
+               (fun () -> !queries, !separator))
+          main))
 
-let () = 
-  SubCommand.register scmd
