@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,59 +20,55 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (** Run a oasis writer helper
     @author Sylvain Le Gall
   *)
 
+
 open OASISGettext
 open OASISQuickstart
 open OASISTypes
-open SubCommand
+open CLISubCommand
 
-let qckstrt_lvl =
-  ref Beginner
 
-let interf = 
-  ref Human
+let main ~ctxt (qckstrt_lvl, interf, setup_data) oasis_fn =
+  let post_run () =
+    let pkg = OASISParse.from_file ~ctxt oasis_fn in
+    Setup.main ~ctxt setup_data oasis_fn pkg
+  in
+  OASISQuickstart.to_file ~ctxt oasis_fn qckstrt_lvl interf post_run
 
-let main () =
-  OASISQuickstart.to_file 
-    ~ctxt:!BaseContext.default
-    !ArgCommon.oasis_fn
-    !qckstrt_lvl
-    !interf
-    Setup.main
 
-let scmd =
+let fspecs () =
+  let (specs, anon), gen = Setup.fspecs () in
   let lvls =
     [
-      s_ "beginner", Beginner; 
-      s_ "intermediate", Intermediate; 
-      s_ "expert", Expert;
+      "beginner", Beginner;
+      "intermediate", Intermediate;
+      "expert", Expert;
     ]
   in
-    {(SubCommand.make
-        ~std_usage:true
-        "quickstart"
-        (s_ "Launch an helper to write `_oasis` file")
-        CLIData.quickstart_mkd
-        main)
-       with 
-           scmd_specs =
-             (
-               "-level",
-               Arg.Symbol
-                 ((List.map fst lvls),
-                  (fun s -> qckstrt_lvl := List.assoc s lvls)),
-               (s_ " Quickstart level, skip questions according to this level.")
-             ) 
-             ::
-             (
-               "-machine",
-               Arg.Unit (fun () -> interf := Machine), 
-               (s_ " Computer readable questions for automatic completion.")
-             )
-             :: SetupDev.scmd.scmd_specs}
+  let qckstrt_lvl = ref Beginner in
+  let interf = ref Human in
+  let specs' =
+    ["-level",
+     Arg.Symbol
+       ((List.map fst lvls),
+        (fun s -> qckstrt_lvl := List.assoc s lvls)),
+     (s_ " Quickstart level, skip questions according to this \
+           level.");
+     "-machine",
+     Arg.Unit (fun () -> interf := Machine),
+     (s_ " Computer readable questions for automatic completion.")] @ specs
+  in
+  let gen' () = !qckstrt_lvl, !interf, gen () in
+  (specs', anon), gen'
 
-let () = 
-  SubCommand.register scmd
+
+let () =
+  CLISubCommand.register "quickstart"
+    (ns_ "Launch an helper to write `_oasis` file")
+    CLIData.quickstart_mkd
+    (CLICommon.define_oasis_fn
+       (CLISubCommand.make_run fspecs main))
