@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,10 +20,12 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (** AST manipulation
     @author Sylvain Le Gall
   *)
 
+
 open OASISTypes
 open OASISUtils
 open OASISGettext
@@ -30,41 +33,20 @@ open OASISRecDescParser
 open OASISAstTypes
 open OASISExpr
 
-(** Convert OASIS stream into package 
+
+(** Convert OASIS stream into package
   *)
-let to_package conf st = 
+let to_package conf st =
 
-  let ast = 
-    OASISRecDescParser.parse_stream 
-      conf 
+  let ast =
+    OASISRecDescParser.parse_stream
+      conf
       st
   in
 
-  let default_ctxt =
-    {
-      cond        = None;
-      append      = false;
-      valid_flags = [];
-      ctxt        = conf.OASISRecDescParser.ctxt;
-    }
-  in
-
-  (* Convert flags into ctxt *)
-  let ctxt_of_sections scts =
-    {default_ctxt with 
-         valid_flags = 
-           (List.fold_left 
-              (fun acc ->
-                function 
-                  | Flag (cs, _) -> cs.cs_name :: acc
-                  | _ -> acc)
-              default_ctxt.valid_flags
-              scts)}
-  in
-
   (* Merge an expression with a condition in a ctxt *)
   let ctxt_add_expr ctxt e =
-    match ctxt with 
+    match ctxt with
       | {cond = None} ->
           {ctxt with cond = Some e}
       | {cond = Some e'} ->
@@ -76,25 +58,25 @@ let to_package conf st =
    *)
   let rec stmt schm data ctxt =
     function
-      | SField (nm, op) -> 
+      | SField (nm, op) ->
           begin
             try
               match op with
                 | FSet s ->
                     begin
-                      PropList.Schema.set 
-                        schm 
-                        data 
-                        nm 
+                      PropList.Schema.set
+                        schm
+                        data
+                        nm
                         ~context:{ctxt with append = false}
                         s
                     end
                 | FAdd s ->
                     begin
-                      PropList.Schema.set 
-                        schm 
-                        data 
-                        nm 
+                      PropList.Schema.set
+                        schm
+                        data
+                        nm
                         ~context:{ctxt with append = true}
                         s
                     end
@@ -106,15 +88,15 @@ let to_package conf st =
                         nm
                         ~context:{ctxt with append = false}
                         (string_of_bool false);
-                      PropList.Schema.set 
-                        schm 
+                      PropList.Schema.set
+                        schm
                         data
-                        nm 
+                        nm
                         ~context:{(ctxt_add_expr ctxt e) with append = false}
                         (string_of_bool true)
                     end
             with (PropList.Unknown_field _) as exc ->
-              if OASISPlugin.test_field_name nm && 
+              if OASISPlugin.test_field_name nm &&
                  ctxt.ctxt.OASISContext.ignore_plugins then
                 ()
               else if ctxt.ctxt.OASISContext.ignore_unknown_fields then
@@ -123,20 +105,20 @@ let to_package conf st =
                 raise exc
           end
 
-      | SIfThenElse (e, stmt1, stmt2) -> 
+      | SIfThenElse (e, stmt1, stmt2) ->
           begin
             (* Check that we have a valid expression *)
             OASISExpr.check ctxt.valid_flags e;
             (* Explore if branch *)
-            stmt 
+            stmt
               schm
               data
               (ctxt_add_expr ctxt e)
               stmt1;
             (* Explore then branch *)
-            stmt 
+            stmt
               schm
-              data 
+              data
               (ctxt_add_expr ctxt (ENot e))
               stmt2
           end
@@ -148,81 +130,54 @@ let to_package conf st =
   (* Explore statement and register data into a newly created
    * Schema.writer.
    *)
-  let schema_stmt gen nm schm oasis_version scts stmt' =
-    let data =
-      PropList.Data.create ()
-    in
-    let ctxt =
-      ctxt_of_sections scts
-    in
-    let schm = 
-      schm.OASISSchema_intern.schm 
-    in
+  let schema_stmt gen nm schm (ctxt, scts) stmt' =
+    let data = PropList.Data.create () in
+    let schm = schm.OASISSchema_intern.schm in
+    let where = (PropList.Schema.name schm)^" "^nm in
       stmt schm data ctxt stmt';
-      OASISCheck.check_schema ~ctxt:ctxt.ctxt 
-        ((PropList.Schema.name schm)^" "^nm)
-        schm
-        oasis_version
-        data;
-      (gen oasis_version nm data) :: scts
+      ctxt,
+      (schm, where, data,
+       fun features_data -> gen features_data nm data)
+      :: scts
   in
 
-  (* Extract and cache oasis_version *)
-  let oasis_version =
-    let rver = ref None in
-      fun data ->
-        try
-          let nver =
-            OASISPackage_intern.oasis_version data
-          in
-          let () =
-            match !rver with
-              | Some ver ->
-                  if nver <> ver then
-                    failwithf
-                      (f_ "Multiple definition of OASISFormat (%s and %s)")
-                      (OASISVersion.string_of_version ver)
-                      (OASISVersion.string_of_version nver);
-              | None ->
-                  rver := Some nver
-          in
-            nver
-        with PropList.Not_set _ ->
-          failwith (s_ "OASISFormat not defined at the beginning of the file, \
-                        consider starting with 'OASISFormat: ...'")
-  in
-
-  (* Recurse into top-level statement. At this level there is 
-   * no conditional expression but there is Flag, Library and
-   * Executable structure defined.
-   *) 
-  let rec top_stmt pkg_data acc =
+  (* Recurse into top-level statement. At this level there is no conditional
+   * expression but there is Flag, Library and Executable structure defined.
+   *)
+  let rec top_stmt pkg_data (ctxt, scts as acc) =
+    (* TODO: refactor all section into one common function call. *)
     function
-      | TSLibrary (nm, stmt) -> 
-          schema_stmt 
+      | TSLibrary (nm, stmt) ->
+          schema_stmt
             OASISLibrary_intern.generator
             nm
-            OASISLibrary.schema 
-            (oasis_version pkg_data)
+            OASISLibrary.schema
             acc
             stmt
 
-      | TSExecutable (nm, stmt) -> 
+      | TSObject (nm, stmt) ->
+          schema_stmt
+            OASISObject_intern.generator
+            nm
+            OASISObject.schema
+            acc
+            stmt
+
+      | TSExecutable (nm, stmt) ->
           schema_stmt
             OASISExecutable_intern.generator
             nm
             OASISExecutable.schema
-            (oasis_version pkg_data)
             acc
             stmt
 
-      | TSFlag (nm, stmt) -> 
-          schema_stmt 
-            OASISFlag_intern.generator 
+      | TSFlag (nm, stmt) ->
+          let ctxt = {ctxt with valid_flags = nm :: ctxt.valid_flags} in
+          schema_stmt
+            OASISFlag_intern.generator
             nm
-            OASISFlag.schema 
-            (oasis_version pkg_data)
-            acc
+            OASISFlag.schema
+            (ctxt, scts)
             stmt
 
       | TSSourceRepository (nm, stmt) ->
@@ -230,7 +185,6 @@ let to_package conf st =
             OASISSourceRepository_intern.generator
             nm
             OASISSourceRepository.schema
-            (oasis_version pkg_data)
             acc
             stmt
 
@@ -239,60 +193,93 @@ let to_package conf st =
             OASISTest_intern.generator
             nm
             OASISTest.schema
-            (oasis_version pkg_data)
             acc
             stmt
 
       | TSDocument (nm, stmt) ->
-          schema_stmt 
+          schema_stmt
             OASISDocument_intern.generator
             nm
             OASISDocument.schema
-            (oasis_version pkg_data)
             acc
             stmt
 
-      | TSStmt stmt' -> 
-          stmt 
+      | TSStmt stmt' ->
+          stmt
             OASISPackage.schema.OASISSchema_intern.schm
-            pkg_data 
-            (ctxt_of_sections acc) 
+            pkg_data
+            ctxt
             stmt';
           acc
 
-      | TSBlock blk -> 
-          List.fold_left 
-            (top_stmt pkg_data) 
-            acc 
+      | TSBlock blk ->
+          List.fold_left
+            (top_stmt pkg_data)
+            acc
             blk
   in
 
-  (* Start with package schema/writer *)
-  let data =
-    PropList.Data.create ()
+  (* Interpret AST and inject it into data. *)
+  let data = PropList.Data.create () in
+  let ctxt, sections =
+    let default_ctxt =
+      {
+        cond        = None;
+        append      = false;
+        valid_flags = [];
+        ctxt        = conf.OASISRecDescParser.ctxt;
+      }
+    in
+    top_stmt data (default_ctxt, []) ast
   in
-  let sections =
-    top_stmt 
-      data
-      []
-      ast
+
+
+  (* Extract features_data *)
+  let features_data =
+    let oasis_version =
+      try
+        OASISPackage_intern.oasis_version data
+      with PropList.Not_set _ ->
+        failwith (s_ "OASISFormat not defined at the beginning of the file, \
+                      consider starting with 'OASISFormat: ...'")
+    in
+      OASISFeatures.Data.create oasis_version
+        (OASISPackage_intern.alpha_features data)
+        (OASISPackage_intern.beta_features data)
   in
-  let pkg = 
-    OASISCheck.check_schema ~ctxt:default_ctxt.ctxt
-      "package"
-      OASISPackage.schema.OASISSchema_intern.schm
-      (oasis_version data)
-      data;
-    OASISPackage_intern.generator 
-      data
-      sections
+
+  (* Check all schema. *)
+  let () =
+    let plugins =
+      OASISCheck.check_schema ~ctxt:ctxt.ctxt
+        "package"
+        OASISPackage.schema.OASISSchema_intern.schm
+        OASISPlugin.SetPlugin.empty
+        features_data
+        data
+    in
+      List.iter
+        (fun (schm, where, data, _) ->
+           let _plugins: OASISPlugin.SetPlugin.t =
+             OASISCheck.check_schema ~ctxt:ctxt.ctxt
+               where schm plugins features_data data
+           in
+             ())
+        sections
+  in
+
+  let pkg =
+    let sections =
+      List.map (fun (_, _, _, gen) -> gen features_data) sections
+    in
+      OASISPackage_intern.generator data sections
   in
 
   (* Fix build depends to reflect internal dependencies *)
-  let pkg = 
+  let pkg =
     (* Map of findlib name to internal libraries *)
     let _, _, internal_of_findlib =
-      OASISLibrary.findlib_mapping pkg
+      OASISFindlib.findlib_mapping pkg
     in
 
     let map_internal_libraries sct =
@@ -302,7 +289,7 @@ let to_package conf st =
                let is_internal, lnm =
                  try
                    true, internal_of_findlib lnm
-                 with (OASISLibrary.FindlibPackageNotFound _) ->
+                 with (OASISFindlib.FindlibPackageNotFound _) ->
                    false, lnm
                in
                  if is_internal then
@@ -322,7 +309,7 @@ let to_package conf st =
                bd)
     in
 
-    let internal_tools = 
+    let internal_tools =
       List.fold_left
         (fun st ->
            function
@@ -334,8 +321,8 @@ let to_package conf st =
         pkg.sections
     in
 
-    let map_internal_tools sct = 
-      List.map 
+    let map_internal_tools sct =
+      List.map
         (function
            | ExternalTool lnm as bt ->
                begin
@@ -348,25 +335,32 @@ let to_package conf st =
                bt)
     in
 
-    let map_internal sct bs = 
-      {bs with 
-           bs_build_depends = 
+    let map_internal sct bs =
+      {bs with
+           bs_build_depends =
              map_internal_libraries sct bs.bs_build_depends;
-           bs_build_tools = 
+           bs_build_tools =
              map_internal_tools sct bs.bs_build_tools}
     in
 
-      {pkg with 
+      {pkg with
            sections =
-             List.map 
+             List.map
                (function
                   | Library (cs, bs, lib) as sct ->
-                      Library 
-                        (cs, 
-                         map_internal 
+                      Library
+                        (cs,
+                         map_internal
                            sct
                            bs,
                          lib)
+                  | Object (cs, bs, obj) as sct ->
+                      Object
+                        (cs,
+                         map_internal
+                           sct
+                           bs,
+                         obj)
                   | Executable (cs, bs, exec) as sct ->
                       Executable
                         (cs,
@@ -375,11 +369,11 @@ let to_package conf st =
                            bs,
                          exec)
                   | Test (cs, tst) as sct ->
-                      Test 
+                      Test
                         (cs,
-                         {tst with 
-                              test_tools = 
-                                map_internal_tools 
+                         {tst with
+                              test_tools =
+                                map_internal_tools
                                   sct
                                   tst.test_tools})
                   | Doc (cs, doc) as sct ->
@@ -396,11 +390,11 @@ let to_package conf st =
   in
 
   (* Check recursion and re-order library/tools so that build order is
-     respected 
+     respected
    *)
-  let pkg = 
+  let pkg =
     {pkg with sections = OASISBuildSection.build_order pkg}
   in
-    OASISCheck.check_package ~ctxt:default_ctxt.ctxt pkg;
+    OASISCheck.check_package ~ctxt:ctxt.ctxt pkg;
     pkg
 
