@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,21 +20,24 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (* END EXPORT *)
 
+
 open OASISTypes
 open OASISUtils
-
 open OASISSection
 open OASISSection_intern
 
 module G = OASISGraph
 
+
 type extended_kind = [section_kind | `ExternalTool | `FindlibPackage]
 type vertex = extended_kind * string
 
+
 let build_graph pkg =
-  let g : vertex G.t = G.create (2 * List.length pkg.sections) in
+  let g: vertex G.t = G.create (2 * List.length pkg.sections) in
   let sct_of_vrtx = Hashtbl.create (List.length pkg.sections) in
   let ext_of_vrtx = Hashtbl.create 13 in
 
@@ -70,8 +74,10 @@ let build_graph pkg =
     List.iter
       (function
          | InternalLibrary nm ->
-             let dvrtx = G.vertex_of_value g (`Library, nm) in
-               G.add_edge g vrtx dvrtx
+             let dvrtx =
+               try G.vertex_of_value g (`Library, nm)
+               with Not_found -> G.vertex_of_value g (`Object, nm) in
+             G.add_edge g vrtx dvrtx
          | FindlibPackage (fndlb_nm, ver_opt) ->
              let dvrtx = G.add_vertex g (`FindlibPackage, fndlb_nm) in
                Hashtbl.add ext_of_vrtx dvrtx
@@ -85,6 +91,7 @@ let build_graph pkg =
       (fun (vrtx, sct) ->
          match sct with
            | Library (cs, bs, _)
+           | Object (cs, bs, _)
            | Executable (cs, bs, _) ->
                add_build_section vrtx bs
            | Test (cs, {test_tools = build_tools})
@@ -96,6 +103,7 @@ let build_graph pkg =
 
     sct_of_vrtx, ext_of_vrtx, g
 
+
 let build_order pkg =
   let sct_of_vrtx, _, g = build_graph pkg in
     List.rev
@@ -108,6 +116,7 @@ let build_order pkg =
          []
          (G.topological_sort g))
 
+
 let transitive_build_depends pkg =
   let sct_of_vrtx, ext_of_vrtx, g = build_graph pkg in
 
