@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,42 +20,48 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (** Global context for messages and i18n
     @author Sylvain Le Gall
   *)
 
+
 type level =
   [ `Debug
-  | `Info 
+  | `Info
   | `Warning
   | `Error]
 
-type t = 
-  { 
+
+type t =
+  {
     quiet: bool;
     (** Display nothing. *)
 
-    info : bool;
+    info: bool;
     (** Display info messages. *)
 
-    debug : bool;   
+    debug: bool;
     (** Display also debug messages. *)
 
-    ignore_plugins : bool;
+    ignore_plugins: bool;
     (** Don't use plugins *)
 
     ignore_unknown_fields: bool;
     (** Ignore fields unknown *)
 
-    printf: level -> string -> unit; 
+    printf: level -> string -> unit;
     (** Redirect output *)
   }
 
+
 (** Default context *)
 val default: t ref
 
-(** Command line arguments to change {!default}. *)
-val args: unit -> (string * Arg.spec * string) list
+
+(** Command line arguments to create {!t}. *)
+val fspecs: unit -> ((string * Arg.spec * string) list * (unit -> t))
+
 
 (** Quiet context. *)
-val quiet : t
+val quiet: t
