@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,34 +20,38 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (** Test schema and generator
     @author Sylvain Le Gall
   *)
 
+
 (* END EXPORT *)
 
+
 open OASISTypes
 open OASISSchema_intern
 open OASISValues
 open OASISUtils
 open OASISGettext
 
+
 let schema, generator =
   let schm =
     schema "Document" (fun (cs, _) -> cs.cs_plugin_data)
   in
   let cmn_section_gen =
-    OASISSection_intern.section_fields 
-      (fun () -> (s_ "document")) schm 
+    OASISSection_intern.section_fields
+      (fun () -> (s_ "document")) schm
       (fun (cs, _) -> cs)
   in
-  let build_tools = 
+  let build_tools =
     OASISBuildSection_intern.build_tools_field schm
       (fun (_, doc) -> doc.doc_build_tools)
   in
   let typ =
     new_field_plugin schm "Type"
-      ~default:(OASISPlugin.builtin `Doc "none") 
+      ~default:(OASISPlugin.builtin `Doc "none")
       `Doc
       OASISPlugin.Doc.value
       (fun () ->
@@ -85,7 +90,7 @@ let schema, generator =
   let doc_format =
     new_field schm "Format"
       ~default:OtherDoc
-      (choices 
+      (choices
          (fun () -> "document format")
          ["HTML",       HTML "index.html";
           "Text",       DocText;
@@ -105,8 +110,8 @@ let schema, generator =
       (fun () ->
          s_ "Index or top-level file for the document, only apply to \
              HTML and Info.")
-      (fun (_, doc) -> 
-         match doc.doc_format with 
+      (fun (_, doc) ->
+         match doc.doc_format with
            | HTML idx | Info idx ->
                Some idx
            | DocText | PDF | PostScript | DVI | OtherDoc ->
@@ -120,21 +125,21 @@ let schema, generator =
          s_ "Default target directory to install data and documentation.")
       (fun (_, doc) -> Some doc.doc_install_dir)
   in
-  let build, install, data_files = 
+  let build, install, data_files =
    OASISBuildSection_intern.build_install_data_fields schm
      (fun (_, doc) -> doc.doc_build)
      (fun (_, doc) -> doc.doc_install)
      (fun (_, doc) -> doc.doc_data_files)
   in
     schm,
-    (fun oasis_version nm data ->
-       let cs = 
-         cmn_section_gen oasis_version nm data
+    (fun features_data nm data ->
+       let cs =
+         cmn_section_gen features_data nm data
        in
-       let typ = 
+       let typ =
          typ data
        in
-       let rplugin_data = 
+       let rplugin_data =
          ref cs.cs_plugin_data
        in
        let cs =
@@ -146,7 +151,7 @@ let schema, generator =
          {cs with cs_plugin_data = !rplugin_data}
        in
        let build =
-         if OASISVersion.version_0_3_or_after oasis_version then
+         if OASISFeatures.data_test OASISFeatures.flag_docs features_data then
            (* TODO: establish a formal link between here and BaseStandardVars *)
            OASISExpr.if_then_else
              (OASISExpr.EFlag "docs") (build data) [OASISExpr.EBool true, false]
@@ -155,11 +160,11 @@ let schema, generator =
        in
          Doc
            (cs,
-            (* TODO: find a way to code that in a way compatible with 
+            (* TODO: find a way to code that in a way compatible with
              * quickstart
              *)
             let doc_format =
-              match doc_format data with 
+              match doc_format data with
                 | HTML _ ->
                     begin
                       match index data with
