@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,10 +20,12 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (** Executable schema and generator
     @author Sylvain Le Gall
   *)
 
+
 open OASISTypes
 open OASISSchema_intern
 open OASISValues
@@ -30,22 +33,23 @@ open OASISUtils
 open OASISGettext
 open PropList.Field
 
+
 let schema, generator =
   let schm =
-    schema "Executable" (fun (cs, _, _) -> cs.cs_plugin_data) 
+    schema "Executable" (fun (cs, _, _) -> cs.cs_plugin_data)
   in
   let cmn_section_gen =
-    OASISSection_intern.section_fields 
+    OASISSection_intern.section_fields
       (fun () -> (s_ "executable")) schm
       (fun (cs, _, _) -> cs)
   in
   let build_section_gen =
-    OASISBuildSection_intern.section_fields 
+    OASISBuildSection_intern.section_fields
       (fun () -> (s_ "executable")) Byte schm
       (fun (_, bs, _) -> bs)
   in
   let main_is =
-    new_field schm "MainIs" 
+    new_field schm "MainIs"
       (let base_value =
          {
            parse =
@@ -60,13 +64,13 @@ let schema, generator =
          }
        in
          {
-           parse  = (fun ~ctxt str -> 
-                       file.parse ~ctxt 
+           parse  = (fun ~ctxt str ->
+                       file.parse ~ctxt
                          (base_value.parse ~ctxt str));
            update = update_fail;
            print  = (fun fn -> file.print (base_value.print fn));
          })
-      (fun () -> 
+      (fun () ->
          s_ "OCaml file (.ml) containing main procedure for the executable.")
       (fun (_, _, exec) -> exec.exec_main_is)
   in
@@ -79,9 +83,9 @@ let schema, generator =
       (fun (_, _, exec) -> exec.exec_custom)
   in
     schm,
-    (fun oasis_version nm data ->
+    (fun features_data nm data ->
        Executable
-         (cmn_section_gen oasis_version nm data,
+         (cmn_section_gen features_data nm data,
           build_section_gen nm data,
           {
             exec_main_is = main_is data;
