@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,14 +20,19 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 TYPE_CONV_PATH "OASISExpr"
 
+
 open OASISGettext
 
+
 type test = string with odn
 
+
 type flag = string with odn
 
+
 type t =
   | EBool of bool
   | ENot of t
@@ -36,8 +42,10 @@ type t =
   | ETest of test * string
   with odn
 
+
 type 'a choices = (t * 'a) list with odn
 
+
 let eval var_get t =
   let rec eval' =
     function
@@ -68,6 +76,7 @@ let eval var_get t =
   in
     eval' t
 
+
 let choose ?printer ?name var_get lst =
   let rec choose_aux =
     function
@@ -104,10 +113,13 @@ let choose ?printer ?name var_get lst =
   in
     choose_aux (List.rev lst)
 
+
 (* END EXPORT *)
 
+
 open OASISUtils
 
+
 let tests =
   [
     "os_type";
@@ -117,13 +129,16 @@ let tests =
     "ocaml_version";
   ]
 
+
 let test_of_string str =
   (* TODO: check for correct syntax of str *)
   str
 
+
 let string_of_test t =
   t
 
+
 let check valid_flags =
   let lowercase_eq str1 str2 =
     (String.lowercase str1) = (String.lowercase str2)
@@ -146,6 +161,7 @@ let check valid_flags =
   in
     check_aux valid_flags
 
+
 let rec reduce e =
   let e =
     match e with
@@ -172,9 +188,11 @@ let rec reduce e =
           EBool true
       | ENot (ENot e) ->
           e
-      | (ENot _ | EAnd (_, _) | EOr (_, _) | EFlag _ | ETest (_, _) | (EBool _)) as e ->
+      | (ENot _ | EAnd (_, _) | EOr (_, _) | EFlag _ | ETest (_, _)
+      | (EBool _)) as e ->
           e
 
+
 let reduce_choices choices =
   (* Naive reduction, we only look for exactly the same condition in
    * after one condition. It works but is not complete and not efficient
@@ -203,6 +221,7 @@ let reduce_choices choices =
       []
       (List.map (fun (cond, vl) -> reduce cond, vl) choices)
 
+
 let if_then_else t choices_if choices_else =
   let choices_if' =
     List.rev_map (fun (t', v) -> EAnd (t, t'), v) choices_if
@@ -212,16 +231,17 @@ let if_then_else t choices_if choices_else =
   in
     reduce_choices (List.rev_append choices_else' (List.rev choices_if'))
 
+
 let rec to_string =
   function
     | EBool b -> string_of_bool b
     | EFlag nm -> "flag("^nm^")"
     | ETest (nm, vl) -> nm^"("^vl^")"
 
-    | EOr (e1, e2) -> 
+    | EOr (e1, e2) ->
         (to_string e1)^" || "^(to_string e2)
 
-    | ENot (EBool _ | EFlag _ | ETest _ | ENot _ as e) -> 
+    | ENot (EBool _ | EFlag _ | ETest _ | ENot _ as e) ->
         "!"^(to_string e)
     | ENot (EAnd _ | EOr _ as e) ->
         "!("^(to_string e)^")"
@@ -235,6 +255,7 @@ let rec to_string =
     | EAnd (e1, e2) ->
         (to_string e1)^" && "^(to_string e2)
 
-let string_of_choices f lst = 
-  "["^(String.concat "; " 
+
+let string_of_choices f lst =
+  "["^(String.concat "; "
          (List.rev_map (fun (e, vl) -> (to_string e)^" -> "^(f vl)) lst))^"]"
