@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,6 +20,7 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (** Generate files replacing parts of it
 
     This module allow to generate files using template. Each template files
@@ -38,80 +40,101 @@
     @author Sylvain Le Gall
   *)
 
+
 open OASISUnixPath
 
+
 (** {2 Comments} *)
 
+
 (** Comment definition. *)
 type comment
 
-(** .ml comments.
-  *)
+
+(** .ml comments.  *)
 val comment_ml: comment
 
-(** Shell comments.
-  *)
+
+(** Shell comments.  *)
 val comment_sh: comment
 
-(** Makefile comments.
-  *)
+
+(** Makefile comments.  *)
 val comment_makefile: comment
 
-(** OCamlbuild comments.
-  *)
+
+(** OCamlbuild comments.  *)
 val comment_ocamlbuild: comment
 
-(** .bat file comments.
-  *)
+
+(** .bat file comments.  *)
 val comment_bat: comment
 
-(** META file comments.
-  *)
+
+(** META file comments.  *)
 val comment_meta: comment
 
+
+(** Markdown comments. *)
+val comment_markdown: comment
+
+
 (** {2 Template} *)
 
+
 type line = string
 
+
 type body =
     NoBody
   | Body of line list
   | BodyWithDigest of Digest.t * line list
 
+
 type template =
     {
-      fn : host_filename;
-      comment : comment;
-      header : line list;
-      body : body;
-      footer : line list;
-      perm : int;
+      fn: host_filename;
+      comment: comment;
+      header: line list;
+      body: body;
+      footer: line list;
+      perm: int;
+      important: bool; (** Determine if should be kept in dynamic mode. *)
+      disable_oasis_section: bool;
+      (** Determine if OASIS section comments and digest should be omitted. *)
     }
 
+
 (** [template_make fn cmt header body footer] Create a template for which
-    target file is [fn].
-  *)
-val template_make :
+    target file is [fn].  *)
+val template_make:
   host_filename ->
   comment -> line list -> line list -> line list -> template
 
-(** [template_of_string_list ~ctxt ~template fn cmt lst] Split a the
-    list [lst] of into an header, a body and a footer, using comment
-    [cmt] to determine each part. Set [~template] if this is an
-    embedded template (i.e. not a file loaded from disk). See
+
+(** [template_of_string_list ~ctxt ~template ~pure fn cmt lst] Split the list
+    [lst] into a header, body and footer, using comment [cmt] to determine each
+    part. Set [~template] if this is an embedded template (i.e. not a file
+    loaded from disk). If [~disable_oasis_section] is set, then the list is
+    processed on the assumption that there is no header and footer. See
     {!template_make} for other options.  *)
-val template_of_string_list :
-  ctxt:OASISContext.t ->
-  template:bool -> host_filename -> comment -> line list -> template
+val template_of_string_list:
+  ctxt:OASISContext.t -> template:bool ->
+  ?disable_oasis_section:bool -> host_filename -> comment -> line list -> template
+
 
 (** [template_of_ml_file fn] Create an OCaml file template taking into account
     subtleties, like line modifier. See {!template_make} for other options.
 *)
-val template_of_mlfile :
+val template_of_mlfile:
   host_filename -> line list -> line list -> line list -> template
 
+
 (** {2 File generation} *)
 
+(** Create a list representation of the file. *)
+val to_string_list: template -> line list
+
 (** Describe what has been done to generate a file out of a template.
   *)
 type file_generate_change =
@@ -123,48 +146,61 @@ type file_generate_change =
   | NoChange
   (** Nothing done, the file doesn't need to be updated *)
 
+
 (** Reset to pristine a generated file.
   *)
-val file_rollback : ctxt:OASISContext.t -> file_generate_change -> unit
+val file_rollback: ctxt:OASISContext.t -> file_generate_change -> unit
+
 
 (** Generate a file using a template. Only the part between OASIS_START and
     OASIS_STOP will be really replaced if the file exists. If the file doesn't
-    exist use the whole template.
+    exist use the whole template. If [~remove] is [true], then an existing file
+    will be deleted iff the template body is [[]] and the header and footer of
+    the file match the template's (used by the -remove option for setup-clean).
  *)
-val file_generate :
-  ctxt:OASISContext.t -> backup:bool -> template -> file_generate_change
+val file_generate:
+  ctxt:OASISContext.t ->
+  ?remove:bool -> backup:bool -> template -> file_generate_change
 
 
 (** {2 Multiple templates management } *)
 
+
 (** Try to add a file that is already in the set
   *)
 exception AlreadyExists of host_filename
 
+
 (** Set of templates.
   *)
 type templates
 
-(** No generated template files.
-  *)
-val empty : templates
+
+(** No generated template files with the given set of files with the OASIS
+    section disabled. *)
+val create: disable_oasis_section:unix_filename list -> unit -> templates
+
 
 (** Find a generated template file.
   *)
-val find : host_filename -> templates -> template
+val find: host_filename -> templates -> template
+
 
 (** Add a generated template file.
   *)
-val add : template -> templates -> templates
+val add: template -> templates -> templates
+
 
 (** Remove a generated template file.
   *)
-val remove : host_filename -> templates -> templates
+val remove: host_filename -> templates -> templates
+
 
 (** Add or replace a generated template file.
   *)
-val replace : template -> templates -> templates
+val replace: template -> templates -> templates
+
 
 (** Fold over generated template files.
   *)
-val fold : (template -> 'b -> 'b) -> templates -> 'b -> 'b
+val fold: (template -> 'b -> 'b) -> templates -> 'b -> 'b
