@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,12 +20,15 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (** Flag schema and generator
     @author Sylvain Le Gall
   *)
 
+
 (* END EXPORT *)
 
+
 open OASISTypes
 open OASISSchema_intern
 open OASISValues
@@ -32,26 +36,27 @@ open OASISUtils
 open OASISGettext
 open PropList.Field
 
-let schema, generator = 
+
+let schema, generator =
   let schm =
     schema "Flag" (fun (cs, _) -> cs.cs_plugin_data)
   in
   let cmn_section_gen =
-    OASISSection_intern.section_fields 
-      (fun () -> (s_ "flag")) 
+    OASISSection_intern.section_fields
+      (fun () -> (s_ "flag"))
       schm
       (fun (cs, _) -> cs)
   in
-  let descr = 
-    new_field schm "Description" 
-      ~default:None 
+  let descr =
+    new_field schm "Description"
+      ~default:None
       (opt string_not_empty)
-      (fun () -> 
+      (fun () ->
          s_ "Help for the flag")
       (fun (_, flag) -> flag.flag_description)
   in
-  let default = 
-    new_field_conditional schm "Default" 
+  let default =
+    new_field_conditional schm "Default"
       ~default:true
       boolean
       (fun () ->
@@ -59,9 +64,9 @@ let schema, generator =
       (fun (_, flag) -> flag.flag_default)
   in
     schm,
-    (fun oasis_version nm data ->
+    (fun features_data nm data ->
        Flag
-         (cmn_section_gen oasis_version nm data,
+         (cmn_section_gen features_data nm data,
           {
             flag_description = descr data;
             flag_default     = default data;
