@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,36 +20,42 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (** Library section
     @author Sylvain Le Gall
   *)
 
+
 open OASISTypes
 
-type library_name = name
-type findlib_part_name = name
-type 'a map_of_findlib_part_name = 'a OASISUtils.MapString.t
 
-exception InternalLibraryNotFound of library_name
-exception FindlibPackageNotFound of findlib_name
+(** Looks for a module file, considering capitalization or not. *)
+val find_module:
+  (string -> bool) ->
+  build_section ->
+  OASISUnixPath.unix_filename ->
+  [ `No_sources of OASISUnixPath.unix_filename list
+  | `Sources of OASISUnixPath.unix_filename * string list ]
+
 
 (** [source_unix_files (cs, bs, lib) source_file_exists] Source files for this
     library. The first part of the tuple is the file without extenstion for
     modules and the second part is the source files matching (e.g. .ml and
     .mli).
   *)
-val source_unix_files :
+val source_unix_files:
   ctxt:OASISContext.t ->
   common_section * build_section * library ->
   (unix_filename -> bool) ->
   (unix_filename * (unix_filename list)) list
 
+
 (** [generated_unix_files ~ctxt source_file_exists has_native_dynlink
     is_native ext_lib ext_dll (cs, bs, lib)]
     Compute all files expected by a build of the library. For each file a list
     of alternatives is provided.
   *)
-val generated_unix_files :
+val generated_unix_files:
   ctxt:OASISContext.t ->
   is_native:bool ->
   has_native_dynlink:bool ->
@@ -58,37 +65,7 @@ val generated_unix_files :
   common_section * build_section * library ->
   unix_filename list list
 
-(** Library groups are organized in trees.
-  *)
-type group_t =
-  | Container of findlib_part_name * group_t list
-  | Package of (findlib_part_name *
-                common_section * 
-                build_section * 
-                library * 
-                group_t list)
-
-(** Compute groups of libraries, associate root libraries with 
-    a tree of its children. A group of libraries is defined by 
-    the fact that these libraries have a parental relationship 
-    and must be installed together, with the same META file.
-  *)
-val findlib_mapping: package ->
-  group_t list *
-  (library_name -> findlib_name) *
-  (findlib_name -> library_name)
-
-(** Return the findlib root name of a group, it takes into account
-    containers. So the return group name is the toplevel name
-    for both libraries and theirs containers.
-  *)
-val findlib_of_group : group_t -> findlib_name
-
-(** Return the root library, i.e. the first found into the group tree
-    that has no parent.
-  *)
-val root_of_group : group_t -> common_section * build_section * library
 
 (** Schema for the section. {b Not exported}.
   *)
-val schema : (common_section * build_section * library) OASISSchema.t
+val schema: (common_section * build_section * library) OASISSchema.t
