@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,43 +20,46 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
-(** Library schema and generator 
+
+(** Library schema and generator
     @author Sylvain Le Gall
   *)
 
+
 open OASISTypes
 open OASISSchema_intern
 open OASISValues
 open OASISGettext
 open PropList.Field
 
+
 let schema, generator =
   let schm =
     schema "Library" (fun (cs, _, _) -> cs.cs_plugin_data)
   in
   let cmn_section_gen =
-    OASISSection_intern.section_fields 
-      (fun () -> (s_ "library")) 
+    OASISSection_intern.section_fields
+      (fun () -> (s_ "library"))
       schm
       (fun (cs, _, _) -> cs)
   in
   let build_section_gen =
-    OASISBuildSection_intern.section_fields 
-      (fun () -> (s_ "library")) 
-      Best 
+    OASISBuildSection_intern.section_fields
+      (fun () -> (s_ "library"))
+      Best
       schm
       (fun (_, bs, _) -> bs)
   in
   let external_modules =
-    new_field schm "Modules" 
+    new_field schm "Modules"
       ~default:[]
       ~quickstart_level:Beginner
       modules
       (fun () ->
-         s_ "List of modules to compile.") 
+         s_ "List of modules to compile.")
       (fun (_, _, lib) -> lib.lib_modules)
   in
-  let internal_modules = 
+  let internal_modules =
     new_field schm "InternalModules"
       ~default:[]
       ~quickstart_level:Beginner
@@ -68,7 +72,7 @@ let schema, generator =
     new_field schm "Pack"
       ~default:false
       ~quickstart_level:Expert
-      ~since_version:"0.3"
+      ~feature:OASISFeatures.pack
       boolean
       (fun () ->
          s_ "Set if we should build a packed library.")
@@ -83,11 +87,11 @@ let schema, generator =
              will be built as its parents and installed along it.")
       (fun (_, _, lib) -> lib.lib_findlib_parent)
   in
-  let findlib_name = 
+  let findlib_name =
     new_field schm "FindlibName"
       ~default:None
       (* TODO: Check that the name is correct if this value is None, the
-               package name must be correct 
+               package name must be correct
        *)
       (opt findlib_name)
       (fun () ->
@@ -97,7 +101,7 @@ let schema, generator =
   let findlib_containers =
     new_field schm "FindlibContainers"
       ~default:[]
-      (* TODO: check that a container doesn't overwrite a real package 
+      (* TODO: check that a container doesn't overwrite a real package
        *)
       (dot_separated string_not_empty)
       (fun () ->
@@ -105,9 +109,9 @@ let schema, generator =
       (fun (_, _, lib) -> lib.lib_findlib_containers)
   in
     schm,
-    (fun oasis_version nm data ->
+    (fun features_data nm data ->
        Library
-         (cmn_section_gen oasis_version nm data,
+         (cmn_section_gen features_data nm data,
           (build_section_gen nm data),
           {
             lib_modules            = external_modules data;
