@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,36 +20,43 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
-(** Parse '_oasis' file 
+
+(** Parse '_oasis' file
 
     The whole module is {b not exported}.
 
     @author Sylvain Le Gall
-  *)  
+  *)
+
 
 open OASISTypes
 
+(** Default name of _oasis. *)
+val default_oasis_fn: host_filename
+
 (** [from_stream ~ctxt ~ignore_plugins ~fn st] Parse the OASIS file [~fn] and
     check it. If [~ignore_plugins] is set, ignore unknown plugin's fields in the
     file.
   *)
-val from_stream: 
-  ctxt:OASISContext.t -> 
-  ?fn:host_filename -> 
+val from_stream:
+  ctxt:OASISContext.t ->
+  ?fn:host_filename ->
   (char Stream.t) ->
   package
 
+
 (** See {!from_stream}, apply to a filename.
   *)
-val from_file :
-  ctxt:OASISContext.t -> 
-  host_filename -> 
+val from_file:
+  ctxt:OASISContext.t ->
+  host_filename ->
   package
 
+
 (** See {!from_stream}, apply to a string.
   *)
-val from_string :
-  ctxt:OASISContext.t -> 
-  ?fn:host_filename -> 
+val from_string:
+  ctxt:OASISContext.t ->
+  ?fn:host_filename ->
   string ->
   package
