@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,53 +20,59 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 open OASISTypes
 
+
 type 'a t = 'a OASISSchema_intern.t
 
-(** Create field name derived from a plugin 
+
+(** Create field name derived from a plugin
   *)
-let make_field_name (_, nm', _) nm = 
+let make_field_name (_, nm', _) nm =
   "X"^nm'^nm
 
+
 (** Synchronize between plugin datastructure and PropList.Data.t
   *)
-let sync_proxy schema t nm (_, get) sync ds =  
-  try 
-    let t' = 
+let sync_proxy schema t nm (_, get) sync ds =
+  try
+    let t' =
       (* Extract plugin data from datastructure *)
       get (ref (schema.OASISSchema_intern.plugin ds))
     in
       sync ds t'
   with e ->
-    Printf.eprintf 
+    Printf.eprintf
       "Field %S in schema %S: %s\n"
       (make_field_name t nm)
       (PropList.Schema.name schema.OASISSchema_intern.schm)
       (Printexc.to_string e);
     raise OASISValues.Not_printable
 
-let new_field schema t nm ?default ?since_version parse hlp pivot_data sync =
+
+let new_field schema t nm ?default ?feature parse hlp pivot_data sync =
   let plugin =
     OASISPlugin.to_plugin t
   in
-    OASISSchema_intern.new_field 
+    OASISSchema_intern.new_field
       schema
-      (make_field_name plugin nm) 
+      (make_field_name plugin nm)
       ?default
-      ?since_version
+      ?feature
       ~plugin
       parse
-      hlp 
+      hlp
       (sync_proxy schema plugin nm pivot_data sync)
 
+
 let new_field_conditional
       schema
       t
       nm
       ?default_cond
       ?default
-      ?since_version
+      ?feature
       parse
       hlp
       pivot_data
@@ -73,14 +80,14 @@ let new_field_conditional
   let plugin =
     OASISPlugin.to_plugin t
   in
-    OASISSchema_intern.new_field_conditional 
+    OASISSchema_intern.new_field_conditional
       schema
-      (make_field_name plugin nm) 
+      (make_field_name plugin nm)
       ?default_cond
-      ?default 
-      ?since_version
+      ?default
+      ?feature
       ~plugin
-      parse 
+      parse
       hlp
       (sync_proxy schema plugin nm pivot_data sync)
 
