@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,10 +20,12 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (** SourceRepository schema and generator (internal)
     @author Sylvain Le Gall
   *)
 
+
 open OASISSchema_intern
 open OASISValues
 open OASISUtils
@@ -30,27 +33,28 @@ open OASISGettext
 open PropList.Field
 open OASISTypes
 
+
 let schema, generator =
   let schm =
     schema "SourceRepository" (fun (cs, _) -> cs.cs_plugin_data)
   in
   let cmn_section_gen =
-    OASISSection_intern.section_fields 
-      (fun () -> (s_ "source repository")) 
+    OASISSection_intern.section_fields
+      (fun () -> (s_ "source repository"))
       schm
       (fun (cs, _) -> cs)
   in
-  let typ = 
+  let typ =
     new_field schm "Type"
-      (choices 
+      (choices
          (fun () -> s_ "source repository type")
-         ["darcs",    Darcs; 
-          "git",      Git; 
-          "svn",      Svn; 
-          "cvs",      Cvs; 
-          "hg",       Hg; 
-          "bzr",      Bzr; 
-          "arch",     Arch; 
+         ["darcs",    Darcs;
+          "git",      Git;
+          "svn",      Svn;
+          "cvs",      Cvs;
+          "hg",       Hg;
+          "bzr",      Bzr;
+          "arch",     Arch;
           "monotone", Monotone])
       (fun () -> s_ "VCS type.")
       (fun (_, src_repo) -> src_repo.src_repo_type)
@@ -86,17 +90,18 @@ let schema, generator =
   in
   let branch =
     new_field_opt "Branch"
-      (fun () -> 
+      (fun () ->
          s_ "Define a meaningful branch for this repository.")
       (fun (_, src_repo) -> src_repo.src_repo_branch)
   in
   let tag =
     new_field_opt "Tag"
-      (fun () -> 
-         s_ "Identify a state corresponding to this particular package version.")
+      (fun () ->
+         s_ "Identify a state corresponding to this particular package \
+             version.")
       (fun (_, src_repo) -> src_repo.src_repo_tag)
   in
-  let subdir = 
+  let subdir =
     new_field_opt "Subdir"
       (fun () ->
          s_ "Define the relative path from the root of the repository to the \
@@ -108,9 +113,9 @@ let schema, generator =
      * in src_repo_type
      *)
     schm,
-    (fun oasis_version nm data ->
+    (fun features_data nm data ->
        SrcRepo
-         (cmn_section_gen oasis_version nm data,
+         (cmn_section_gen features_data nm data,
           {
             src_repo_type        = typ data;
             src_repo_location    = location data;
