@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,46 +20,50 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (** Test schema and generator
     @author Sylvain Le Gall
   *)
 
+
 (* END EXPORT *)
 
+
 open OASISTypes
 open OASISSchema_intern
 open OASISValues
 open OASISUtils
 open OASISGettext
 
+
 let schema, generator =
   let schm =
    schema "Test" (fun (cs, _) -> cs.cs_plugin_data)
   in
   let cmn_section_gen =
-    OASISSection_intern.section_fields 
-      (fun () -> (s_ "test")) 
+    OASISSection_intern.section_fields
+      (fun () -> (s_ "test"))
       schm
       (fun (cs, _) -> cs)
   in
   let typ =
     new_field_plugin schm "Type"
-      ~default:(OASISPlugin.builtin `Test "custom") 
+      ~default:(OASISPlugin.builtin `Test "custom")
       `Test
       OASISPlugin.Test.value
       (fun () ->
          s_ "Plugin to use to run test.")
       (fun (_, test) -> test.test_type)
   in
-  let tools = 
+  let tools =
     new_field schm "TestTools"
       ~default:[]
       OASISBuildSection_intern.build_tools_value
-      (fun () -> 
+      (fun () ->
          s_ "Tools required to run the test, including internal executables.")
       (fun (_, test) -> test.test_tools)
   in
-  let command = 
+  let command =
     new_field_conditional schm "Command"
       command_line
       (fun () ->
@@ -66,7 +71,7 @@ let schema, generator =
       (fun (_, test) -> test.test_command)
   in
   let working_directory =
-    new_field schm "WorkingDirectory" 
+    new_field schm "WorkingDirectory"
       ~default:None
       (opt string_not_empty)
       (fun () ->
@@ -79,7 +84,7 @@ let schema, generator =
       (fun () -> s_ "Command to run after the test")
       (fun (_, test) -> test.test_custom)
   in
-  let run = 
+  let run =
     new_field_conditional schm "Run"
       ~default:true
       boolean
@@ -88,9 +93,9 @@ let schema, generator =
       (fun (_, test) -> test.test_run)
   in
     schm,
-    (fun oasis_version nm data ->
-       let cs = 
-         cmn_section_gen oasis_version nm data
+    (fun features_data nm data ->
+       let cs =
+         cmn_section_gen features_data nm data
        in
        (* Set data specific to plugin used for this test *)
        let typ =
@@ -99,8 +104,8 @@ let schema, generator =
        let rplugin_data =
          ref cs.cs_plugin_data
        in
-       let cs = 
-         OASISPlugin.generator_section 
+       let cs =
+         OASISPlugin.generator_section
            `Test
            (typ :> plugin_kind plugin)
            rplugin_data
@@ -108,7 +113,7 @@ let schema, generator =
          {cs with cs_plugin_data = !rplugin_data}
        in
        let run =
-         if OASISVersion.version_0_3_or_after oasis_version then
+         if OASISFeatures.data_test OASISFeatures.flag_tests features_data then
            (* TODO: establish a formal link between here and BaseStandardVars *)
            OASISExpr.if_then_else
              (OASISExpr.EFlag "tests") (run data) [OASISExpr.EBool true, false]
