@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,16 +20,19 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (** Package definition
 
-    This module contains all the types used to build an OASIS package. 
+    This module contains all the types used to build an OASIS package.
     A package is described by an '_oasis' file.
 
     @author Sylvain Le Gall
   *)
 
+
 (** {2 Aliases} *)
 
+
 type name          = string  (** Standard name. *)
 type package_name  = string  (** Name of a package, see {!package}. *)
 type url           = string  (** Standard URL. *)
@@ -41,42 +45,50 @@ type arg           = string  (** Command line argument. *)
 type args          = arg list (** Command line arguments. *)
 type command_line  = (prog * args) (** Command line. *)
 
-(** Package name for findlib, doesn't contain '.'. *)
-type findlib_name = string  
 
-(** Package name + path for findlib, made of several findlib name concatenated with '.'. *)
-type findlib_full = string 
+(** Package name for findlib, doesn't contain '.'. *)
+type findlib_name = string
+
+
+(** Package name + path for findlib, made of several findlib name concatenated
+    with '.'. *)
+type findlib_full = string
+
 
 (** {2 OASIS package definition} *)
 
+
 (** Compilation type.
   *)
 type compiled_object =
-  | Byte   (** Always use byte code. *) 
+  | Byte   (** Always use byte code. *)
   | Native (** Always use native code. *)
   | Best   (** Use either byte or native, depending ocamlopt availability. *)
-  
+
+
 (** Package dependency.
   *)
-type dependency = 
+type dependency =
   | FindlibPackage of findlib_full * OASISVersion.comparator option
     (** External findlib package. *)
 
   | InternalLibrary of name
     (** Section library of the given name. *)
-  
+
+
 (** Tool dependency.
   *)
 type tool =
   | ExternalTool of name
     (** External tool. *)
 
-  | InternalExecutable of name 
+  | InternalExecutable of name
     (** Section executable of the given name. *)
-  
+
+
 (** Possible VCS.
   *)
-type vcs = 
+type vcs =
   | Darcs (** Darcs *)
   | Git   (** Git *)
   | Svn   (** Subversion *)
@@ -86,22 +98,25 @@ type vcs =
   | Arch  (** GNU arch *)
   | Monotone (** Monotone *)
   | OtherVCS of url (** Anything else, follow URL for description. *)
-  
+
+
 (** Conditional value, last expression that evaluate to true define
     the value.
   *)
 type 'a conditional = 'a OASISExpr.choices
 
-(** Plugin kind. 
+
+(** Plugin kind.
   *)
-type plugin_kind = 
-    [`Configure 
-    | `Build 
-    | `Doc 
-    | `Test 
-    | `Install 
+type plugin_kind =
+    [`Configure
+    | `Build
+    | `Doc
+    | `Test
+    | `Install
     | `Extra]
 
+
 (** Additional data to allow registration of more than
     one data property per plugin. See {!OASISPlugin.data_new_property}
   *)
@@ -119,28 +134,33 @@ type plugin_data_purpose =
      | `Other of string
     ]
 
+
 (** Plugin definition, plugin type depends on which fields this
     types is used for.
   *)
-type 'a plugin = 'a * name * OASISVersion.t option 
+type 'a plugin = 'a * name * OASISVersion.t option
+
 
 type all_plugin = plugin_kind plugin
 
+
 (** Property list storage for plugin data
   *)
 type plugin_data = (all_plugin * plugin_data_purpose * (unit -> unit)) list
 
+
 (** Set of command lines to run before and after a step.
   *)
-type custom = 
+type custom =
     {
       pre_command:  (command_line option) conditional;
       (** Run before. *)
 
-      post_command: (command_line option) conditional; 
+      post_command: (command_line option) conditional;
       (** Run after. *)
     }
 
+
 (** Common attributes for sections.
   *)
 type common_section =
@@ -153,7 +173,8 @@ type common_section =
       cs_plugin_data: plugin_data;
       (** Property list attached to the section. *)
     }
-    
+
+
 (** Common attributes for Executable and Library sections.
   *)
 type build_section =
@@ -187,10 +208,11 @@ type build_section =
       bs_nativeopt:       args conditional;
       (** Option to pass to ocamlopt. *)
     }
-    
+
+
 (** Library definition.
   *)
-type library = 
+type library =
     {
       lib_modules:            string list;
       (** List of modules exported by the library. *)
@@ -201,18 +223,31 @@ type library =
       lib_findlib_parent:     findlib_name option;
       (** Name of the findlib parent, if any. *)
       lib_findlib_name:       findlib_name option;
-      (** Findlib name of this library, this name is used to refer to this 
-          library in build dependencies. 
+      (** Findlib name of this library, this name is used to refer to this
+          library in build dependencies.
         *)
       lib_findlib_containers: findlib_name list;
-      (** Name of virtual containes (empty findlib package) between findlib
-          parent and findlib name 
+      (** Name of virtual containers (empty findlib package) between findlib
+          parent and findlib name
         *)
-    } 
+    }
+
+
+(** Object definition.
+  *)
+type object_ =
+    {
+      obj_modules:            string list;
+      obj_findlib_fullname:   findlib_name list option;
+      (** Findlib name of this library, this name is used to refer to this
+          library in build dependencies.
+        *)
+    }
+
 
 (** Executable definition.
   *)
-type executable = 
+type executable =
     {
       exec_custom:          bool;
       (** Use -custom ocamlc option. *)
@@ -220,21 +255,23 @@ type executable =
       (** Main file to compile, dependencies should be guessed
           by build system.
         *)
-    } 
+    }
+
 
 (** Command line flag definition.
   *)
-type flag = 
+type flag =
     {
       flag_description:  string option;
       (** Short description. *)
       flag_default:      bool conditional;
       (** Default value. *)
-    } 
+    }
+
 
 (** Source repository definition.
   *)
-type source_repository = 
+type source_repository =
     {
       src_repo_type:        vcs;
       (** Repository type. *)
@@ -250,11 +287,12 @@ type source_repository =
       (** Depending on VCS, which tag (e.g. tag for darcs, git or svn). *)
       src_repo_subdir:      unix_filename option;
       (** Depending on VCS, which sub directory (e.g. svn). *)
-    } 
+    }
+
 
 (** Test definition.
   *)
-type test = 
+type test =
     {
       test_type:               [`Test] plugin;
       (** Plugin to run the test, default custom. *)
@@ -268,21 +306,22 @@ type test =
       (** Should we run the test. *)
       test_tools:              tool list;
       (** Tools required for this test. *)
-    } 
+    }
+
 
 (** Document formats.
   *)
 type doc_format =
-  | HTML of unix_filename (** HTML files and their main entry point 
-                              (e.g. [HTML "index.html"]) 
+  | HTML of unix_filename (** HTML files and their main entry point
+                              (e.g. [HTML "index.html"])
                             *)
   | DocText               (** Plain text. *)
   | PDF                   (** Portable document format. *)
-  | PostScript       
+  | PostScript
   | Info of unix_filename (** Info files and their main entry point. *)
-  | DVI               
+  | DVI
   | OtherDoc              (** Anything else. *)
-  
+
 
 (** Document definition.
   *)
@@ -312,99 +351,113 @@ type doc =
         *)
       doc_build_tools: tool list;
       (** Tools required to generate this document. *)
-    } 
+    }
+
 
 (** All sections and their attributes. *)
 type section =
   | Library    of common_section * build_section * library
+  | Object     of common_section * build_section * object_
   | Executable of common_section * build_section * executable
   | Flag       of common_section * flag
   | SrcRepo    of common_section * source_repository
   | Test       of common_section * test
   | Doc        of common_section * doc
-  
+
+
 type section_kind =
-    [ `Library | `Executable | `Flag | `SrcRepo | `Test | `Doc ]
+    [ `Library | `Object | `Executable | `Flag | `SrcRepo | `Test | `Doc ]
+
 
 (** OASIS package, what an '_oasis' file contains.
   *)
-type package = 
+type package =
     {
-      oasis_version:    OASISVersion.t;
+      oasis_version:          OASISVersion.t;
       (** OASIS version used to write this package. *)
-      ocaml_version:    OASISVersion.comparator option;
+      ocaml_version:          OASISVersion.comparator option;
       (** OCaml version required for this package. *)
-      findlib_version:  OASISVersion.comparator option;
+      findlib_version:        OASISVersion.comparator option;
       (** Findlib version required for this package. *)
-      name:             package_name;
+      alpha_features:         string list;
+      (** Alpha features enabled for this package. *)
+      beta_features:          string list;
+      (** Beta features enabled for this package. *)
+      name:                   package_name;
       (** Name of this package. *)
-      version:          OASISVersion.t;
+      version:                OASISVersion.t;
       (** Version of this package. *)
-      license:          OASISLicense.t;
+      license:                OASISLicense.t;
       (** License of this package. *)
-      license_file:     unix_filename option;
+      license_file:           unix_filename option;
       (** File containing the license of this package. *)
-      copyrights:       string list;
+      copyrights:             string list;
       (** Copyright holders (e.g. companies or people). *)
-      maintainers:      string list;
+      maintainers:            string list;
       (** People actually taking care of this package (e.g. subset of copyright
           holders)
         *)
-      authors:          string list;
+      authors:                string list;
       (** Real people who wrote this package, not their companies. *)
-      homepage:         url option;
+      homepage:               url option;
       (** Location of the package homepage. *)
-      synopsis:         string;
+      synopsis:               string;
       (** Short description of the package. *)
-      description:      string option;
+      description:            OASISText.t option;
       (** Long description of the package. *)
-      categories:       url list;
+      categories:             url list;
       (** List of categories that the package belong to. *)
 
-      conf_type:        [`Configure] plugin;
+      conf_type:              [`Configure] plugin;
       (** Plugin to configure, default internal. *)
-      conf_custom:      custom;
+      conf_custom:            custom;
       (** Actions around configure step. *)
 
-      build_type:       [`Build] plugin;
+      build_type:             [`Build] plugin;
       (** Plugin to build, default ocamlbuild. *)
-      build_custom:     custom;
+      build_custom:           custom;
       (** Actions around build step. *)
 
-      install_type:     [`Install] plugin;
+      install_type:           [`Install] plugin;
       (** Plugin to install/uninstall, default internal. *)
-      install_custom:   custom;
+      install_custom:         custom;
       (** Actions around install step. *)
-      uninstall_custom: custom;
+      uninstall_custom:       custom;
       (** Actions around uninstall step. *)
 
-      clean_custom:     custom;
+      clean_custom:           custom;
       (** Actions around clean step. *)
-      distclean_custom: custom;
+      distclean_custom:       custom;
       (** Actions aroudn distclean step. *)
 
-      files_ab:         unix_filename list;
+      files_ab:               unix_filename list;
       (** Files to generate by replacing token in it after configure step. *)
-      sections:         section list;
+      sections:               section list;
       (** All sections (libraries, executables, tests...). *)
-      plugins:          [`Extra] plugin list;
+      plugins:                [`Extra] plugin list;
       (** Extra plugins applied. *)
 
+      disable_oasis_section:  unix_filename list;
+      (** Files which should not have OASIS Section comments and digests *)
+
       (* TODO: get rid of schema_data and cs_data *)
-      schema_data:      PropList.Data.t;
-      plugin_data:      plugin_data;
+      schema_data:            PropList.Data.t;
+      plugin_data:            plugin_data;
       (** Property list attached to this package. *)
-    } 
+    }
+
 
 (** {2 Quickstart questions } *)
 
+
 (** Quickstart level. {b Not exported}.
   *)
 type 'a quickstart_level =
   | NoChoice of 'a (** Don't ask question, use provided value. *)
-  | Beginner       (** Ask the question to an OASIS beginner and above. *)
-  | Intermediate   (** Ask the question to an intermediate OASIS user and above. *)
-  | Expert         (** Ask the question to an OASIS expert. *)
+  | Beginner       (** Ask the question to a beginner and above. *)
+  | Intermediate   (** Ask the question to an intermediate user and above. *)
+  | Expert         (** Ask the question to an expert. *)
+
 
 (** Howto ask questions in quickstart. {b Not exported}.
   *)
@@ -414,14 +467,16 @@ type 'a quickstart_question =
   | Choices of 'a list (** Multiple choices in a list. *)
   | ExclusiveChoices of 'a list (** Pick a single choice in the list. *)
 
-(** {2 ODN functions} 
-    
+
+(** {2 ODN functions}
+
     These functions allow to generate setup.ml standalone code, by serializing
     OCaml data using OCaml notation. They are {b not exported}.
 
     See {{:http://forge.ocamlcore.org/projects/odn} the OCaml data notation project}
  *)
 
+
 val odn_of_name:          name -> ODN.t
 val odn_of_package_name:  package_name -> ODN.t
 val odn_of_url:           url -> ODN.t
@@ -433,5 +488,5 @@ val odn_of_args:          args -> ODN.t
 val odn_of_command_line:  prog * args -> ODN.t
 val odn_of_findlib_name:  findlib_name -> ODN.t
 val odn_of_findlib_full:  findlib_name -> ODN.t
-val odn_of_conditional :  ('a -> ODN.t) -> 'a OASISExpr.choices -> ODN.t
-val odn_of_package : package -> ODN.t
+val odn_of_conditional:  ('a -> ODN.t) -> 'a OASISExpr.choices -> ODN.t
+val odn_of_package: package -> ODN.t
