@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,19 +20,25 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 type unix_filename = string
 type unix_dirname = string
 
+
 type host_filename = string
 type host_dirname = string
 
+
 let current_dir_name = "."
 
+
 let parent_dir_name = ".."
 
+
 let is_current_dir fn =
   fn = current_dir_name || fn = ""
 
+
 let concat f1 f2 =
   if is_current_dir f1 then
     f2
@@ -41,6 +48,7 @@ let concat f1 f2 =
     in
       f1'^"/"^f2
 
+
 let make =
   function
     | hd :: tl ->
@@ -51,12 +59,14 @@ let make =
     | [] ->
         invalid_arg "OASISUnixPath.make"
 
+
 let dirname f =
   try
     String.sub f 0 (String.rindex f '/')
   with Not_found ->
     current_dir_name
 
+
 let basename f =
   try
     let pos_start =
@@ -66,6 +76,7 @@ let basename f =
   with Not_found ->
     f
 
+
 let chop_extension f =
   try
     let last_dot =
@@ -88,30 +99,38 @@ let chop_extension f =
   with Not_found ->
     f
 
+
 let capitalize_file f =
   let dir = dirname f in
   let base = basename f in
   concat dir (String.capitalize base)
 
+
 let uncapitalize_file f =
   let dir = dirname f in
   let base = basename f in
   concat dir (String.uncapitalize base)
 
+
 (* END EXPORT *)
 
+
 let check_extension fn ext =
   OASISString.ends_with ~what:("."^ext) fn
 
+
 let add_extension fn ext =
   fn^"."^ext
 
+
 let replace_extension fn ext =
   add_extension (Filename.chop_extension fn) ext
 
+
 open OASISPath_intern
 open OASISUtils
 
+
 let filename_of_list lst =
   let buf = Buffer.create 34 in
     List.iter
@@ -137,9 +156,12 @@ let filename_of_list lst =
       lst;
     Buffer.contents buf
 
+
 let fn_norm fn = fn_reduce [] (fn_reader ~os_type:"Unix" fn)
 
-let reduce fn = filename_of_list  (fn_norm fn)
+
+let reduce fn = filename_of_list (fn_norm fn)
+
 
 let make_relative fn_root fn =
   (* Basic analysis of fn_root and fn. *)
@@ -212,8 +234,17 @@ let make_relative fn_root fn =
       (fn_norm res);
     res
 
+
 let is_current fn =
   if fn = current_dir_name || fn = "" then
     true
   else
     (fn_norm fn) = [`CurrentDir]
+
+
+module Set = SetExt.Make(
+struct
+  type t = unix_filename
+
+  let compare t1 t2 = String.compare (reduce t1) (reduce t2)
+end)
