@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,32 +20,72 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 open OASISGettext
 
-module MapString = Map.Make(String)
 
-let map_string_of_assoc assoc =
-  List.fold_left
-    (fun acc (k, v) -> MapString.add k v acc)
-    MapString.empty
-    assoc
+module MapExt =
+struct
+  module type S =
+  sig
+    include Map.S
+    val add_list: 'a t -> (key * 'a) list -> 'a t
+    val of_list: (key * 'a) list -> 'a t
+    val to_list: 'a t -> (key * 'a) list
+  end
 
-module SetString = Set.Make(String)
+  module Make (Ord: Map.OrderedType) =
+  struct
+    include Map.Make(Ord)
 
-let set_string_add_list st lst =
-  List.fold_left
-    (fun acc e -> SetString.add e acc)
-    st
-    lst
+    let rec add_list t =
+      function
+        | (k, v) :: tl -> add_list (add k v t) tl
+        | [] -> t
 
-let set_string_of_list =
-  set_string_add_list
-    SetString.empty
+    let of_list lst = add_list empty lst
+
+    let to_list t = fold (fun k v acc -> (k, v) :: acc) t []
+  end
+end
+
+
+module MapString = MapExt.Make(String)
+
+
+module SetExt  =
+struct
+  module type S =
+  sig
+    include Set.S
+    val add_list: t -> elt list -> t
+    val of_list: elt list -> t
+    val to_list: t -> elt list
+  end
+
+  module Make (Ord: Set.OrderedType) =
+  struct
+    include Set.Make(Ord)
+
+    let rec add_list t =
+      function
+        | e :: tl -> add_list (add e t) tl
+        | [] -> t
+
+    let of_list lst = add_list empty lst
+
+    let to_list = elements
+  end
+end
+
+
+module SetString = SetExt.Make(String)
 
 
 let compare_csl s1 s2 =
   String.compare (String.lowercase s1) (String.lowercase s2)
 
+
 module HashStringCsl =
   Hashtbl.Make
     (struct
@@ -57,6 +98,14 @@ module HashStringCsl =
          Hashtbl.hash (String.lowercase s)
      end)
 
+module SetStringCsl =
+  SetExt.Make
+    (struct
+       type t = string
+       let compare = compare_csl
+     end)
+
+
 let varname_of_string ?(hyphen='_') s =
   if String.length s = 0 then
     begin
@@ -87,6 +136,7 @@ let varname_of_string ?(hyphen='_') s =
         String.lowercase buf
     end
 
+
 let varname_concat ?(hyphen='_') p s =
   let what = String.make 1 hyphen in
   let p =
@@ -107,35 +157,24 @@ let varname_concat ?(hyphen='_') p s =
 let is_varname str =
   str = varname_of_string str
 
+
 let failwithf fmt = Printf.ksprintf failwith fmt
 
+
 (* END EXPORT *)
 
-let split_comma str =
-  List.map OASISString.trim (OASISString.nsplit str ',')
 
-let split_newline ?(trim=true) str =
-  let lst = OASISString.nsplit str '\n' in
-    if trim then 
-      List.map OASISString.trim lst
-    else 
-      lst
+let may f =
+  function
+    | Some x -> f x
+    | None -> ()
 
-let split_optional_parentheses str =
-  try
-    let beg_str, end_str = OASISString.split (OASISString.trim str) '(' in
-    let content_str = OASISString.strip_ends_with ~what:")" end_str in
-      OASISString.trim beg_str,
-      Some (OASISString.trim content_str)
-  with Not_found ->
-    OASISString.trim str, None
 
 module POSIXShell =
 struct
-
   let unescape s =
     let buf = Buffer.create (String.length s) in
-    let _b : bool =
+    let _b: bool =
       OASISString.fold_left
         (fun escaped_char ->
            function
@@ -192,7 +231,7 @@ struct
     in
 
     (* Protect Buffer.add_substitute substitution inside a string, the $... will
-     * be transformed into $X0, $X1,...
+     * be transformed into $X0, $X1...
      *)
     let substr_data =
       Hashtbl.create 13
@@ -349,5 +388,4 @@ struct
       if last <> "" then rargs := last :: !rargs
     in
     List.rev_map unprotect_subst !rargs
-
 end
