@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,12 +20,15 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 open OASISGettext
 open OASISUtils
 
+
 exception Not_printable
 exception Not_combinable
 
+
 type 'a t =
     {
       parse:  ctxt:OASISContext.t -> string -> 'a;
@@ -32,13 +36,15 @@ type 'a t =
       print:  'a -> string;
     }
 
+
 let update_fail _ _ =
   raise Not_combinable
 
+
 let blackbox =
   {
-    parse  = 
-      (fun ~ctxt s -> 
+    parse  =
+      (fun ~ctxt s ->
          failwithf
            (f_ "Blackbox type cannot be set to the value '%s'")
            s);
@@ -46,6 +52,7 @@ let blackbox =
     print  = (fun _ -> raise Not_printable);
   }
 
+
 module StdLexer =
 struct
   let url = OASISValues_lexer.url
@@ -53,7 +60,8 @@ struct
   let modul = OASISValues_lexer.modul
 end
 
-let lexer lxr nm =
+
+let lexer ?(fail=(fun ~ctxt _ _ -> ())) lxr nm =
   {
     parse =
       (fun ~ctxt str ->
@@ -68,20 +76,23 @@ let lexer lxr nm =
                  str_matched
                  str
                  (nm ())
-         with Not_found ->
+         with e ->
+           fail ~ctxt str e;
+           (* Catch all if the previous ignore error. *)
            failwithf
-             (f_ "String '%s' is not a %s")
-             str
-             (nm ()));
+             (f_ "String '%s' is not a %s: %s.")
+             str (nm ()) (Printexc.to_string e));
     update = update_fail;
     print = (fun s -> s);
   }
 
+
 let url =
   lexer
     StdLexer.url
     (fun () -> s_ "URL")
 
+
 let copyright =
   let base_value =
     lexer
@@ -99,13 +110,15 @@ let copyright =
                        '(C) 2008-2009 J.R. Hacker', here it is '%s'")
                   str)}
 
+
 let string =
-  { 
+  {
     parse =  (fun ~ctxt s -> s);
-    update = (fun s1 s2 -> s1^" "^s2); 
+    update = (fun s1 s2 -> s1^" "^s2);
     print =  (fun s -> s);
   }
 
+
 let string_not_empty =
   {
     parse =
@@ -118,22 +131,26 @@ let string_not_empty =
     print = (fun s -> s);
   }
 
-let file = 
+
+let file =
   {string_not_empty with update = update_fail}
 
+
 let file_glob =
   {string_not_empty with update = update_fail}
 
+
 let directory =
   {string_not_empty with update = update_fail}
 
 
 let expandable value =
-  (* TODO: check expandable value and return a list rather 
+  (* TODO: check expandable value and return a list rather
    * than a single value. Use split_expandable defined above.
    *)
   value
 
+
 let dot_separated value =
   {
     parse =
@@ -141,40 +158,42 @@ let dot_separated value =
          List.map
            (value.parse ~ctxt)
            (OASISString.nsplit s '.'));
-    update = 
+    update =
       List.append;
     print =
       (fun lst ->
-         String.concat "." 
-           (List.map 
+         String.concat "."
+           (List.map
               value.print
               lst));
   }
 
+
 let comma_separated value =
-  { 
-    parse = 
+  {
+    parse =
       (fun ~ctxt s ->
-         List.map 
+         List.map
            (fun s -> value.parse ~ctxt s)
-           (split_comma s));
-    update = 
+           (OASISString.split_comma s));
+    update =
       List.append;
-    print = 
+    print =
       (fun lst ->
          String.concat ", "
-           (List.map 
+           (List.map
               value.print
               lst));
   }
 
+
 let newline_separated value =
   {
     parse =
       (fun ~ctxt s ->
          List.map
            (fun s -> value.parse ~ctxt s)
-           (split_newline s));
+           (OASISString.split_newline s));
     update =
       List.append;
     print =
@@ -185,25 +204,27 @@ let newline_separated value =
               lst));
   }
 
-let space_separated = 
+
+let space_separated =
   {
-    parse = 
+    parse =
       (fun ~ctxt s ->
-         List.filter 
+         List.filter
            (fun s -> s <> "")
            (OASISString.nsplit s ' '));
-    update = 
+    update =
       List.append;
     print =
       (fun lst ->
          String.concat " " lst);
   }
 
+
 let with_optional_parentheses main_value optional_value =
   {
-    parse = 
+    parse =
       (fun ~ctxt str ->
-         match split_optional_parentheses str with 
+         match OASISString.split_optional_parentheses str with
            | e1, Some e2 ->
                main_value.parse ~ctxt e1,
                Some (optional_value.parse ~ctxt e2)
@@ -221,6 +242,7 @@ let with_optional_parentheses main_value optional_value =
                (optional_value.print opt));
   }
 
+
 let opt value =
   {
     parse = (fun ~ctxt str -> Some (value.parse ~ctxt str));
@@ -231,20 +253,30 @@ let opt value =
          | None -> raise Not_printable);
   }
 
+
 let modules =
-  let base_value = 
+  let base_value =
     lexer
       StdLexer.modul
+      ~fail:(fun ~ctxt str e ->
+               match e with
+                 | Failure "lexing: empty token" ->
+                     if String.capitalize str <> str then
+                       failwithf
+                         (f_ "Module name '%s', must be capitalized ('%s').")
+                         str (String.capitalize str)
+                 | _ ->
+                     ())
       (fun () -> s_ "module")
   in
     comma_separated
       {
-        parse = 
+        parse =
          (fun ~ctxt s ->
-            let path = 
+            let path =
               OASISUnixPath.dirname s
             in
-            let modul = 
+            let modul =
               OASISUnixPath.basename s
             in
               if String.contains path ' ' then
@@ -258,20 +290,23 @@ let modules =
         print  = (fun s -> s);
       }
 
-let files = 
+
+let files =
   comma_separated file
 
-let categories = 
-  comma_separated url 
+
+let categories =
+  comma_separated url
+
 
 let choices nm lst =
   {
-    parse = 
+    parse =
       (fun ~ctxt str ->
-         try 
-           List.assoc 
+         try
+           List.assoc
              (String.lowercase str)
-             (List.map 
+             (List.map
                (fun (k, v) ->
                   String.lowercase k, v)
                   lst)
@@ -295,14 +330,16 @@ let choices nm lst =
              (nm ()));
   }
 
+
 let boolean =
-  choices 
+  choices
     (fun () -> s_ "boolean")
     ["true", true; "false", false]
 
+
 let findlib_name =
   {
-    parse = 
+    parse =
       (fun ~ctxt s ->
          if s = "" then
            failwith (s_ "Empty string is not a valid findlib package")
@@ -314,13 +351,14 @@ let findlib_name =
     print = (fun s -> s);
   }
 
+
 let findlib_full =
   {
     parse =
       (fun ~ctxt s ->
          let cpnts = OASISString.nsplit s '.' in
          List.iter (fun cpnt ->
-                    let _s : string = findlib_name.parse ~ctxt cpnt in
+                    let _s: string = findlib_name.parse ~ctxt cpnt in
                     ())
              cpnts;
            s);
@@ -328,20 +366,22 @@ let findlib_full =
     print = (fun s -> s);
   }
 
+
 let internal_library =
   (* TODO: check that the library really exists *)
   {string with update = update_fail}
 
-let command_line = 
+
+let command_line =
   let split_expandable str =
 
     (* Add a single char to accumulator *)
     let rec addchr c =
       function
-        | Some b, _ as acc -> 
+        | Some b, _ as acc ->
             Buffer.add_char b c;
             acc
-        | None, l -> 
+        | None, l ->
             let b =
               Buffer.create 13
             in
@@ -353,7 +393,7 @@ let command_line =
      *)
     let addsep =
       function
-        | Some b, l -> 
+        | Some b, l ->
             None, (Buffer.contents b) :: l
         | None, l ->
             None, l
@@ -365,7 +405,7 @@ let command_line =
     let rec lookup_closing oc cc acc =
       function
         | c :: tl ->
-            let acc = 
+            let acc =
               addchr c acc
             in
               if c = oc then
@@ -381,33 +421,33 @@ let command_line =
                 end
               else
                 begin
-                  lookup_closing oc cc acc tl 
+                  lookup_closing oc cc acc tl
                 end
         | [] ->
             failwithf
               (f_ "'%s' contains unbalanced curly braces")
               str
     in
-    let rec lookup_dollar acc = 
-      function 
+    let rec lookup_dollar acc =
+      function
         | '$' :: ('(' as c) :: tl
-        | '$' :: ('{' as c) :: tl -> 
+        | '$' :: ('{' as c) :: tl ->
             begin
-              let acc, tl = 
-                lookup_closing 
+              let acc, tl =
+                lookup_closing
                   c (if c = '(' then ')' else '}')
                   (addchr c (addchr '$' acc))
                   tl
               in
                 lookup_dollar acc tl
             end
-        | ' ' :: tl -> 
+        | ' ' :: tl ->
             lookup_dollar (addsep acc) tl
-        | c :: tl -> 
+        | c :: tl ->
             lookup_dollar (addchr c acc) tl
         | [] ->
             begin
-              let l = 
+              let l =
                 match acc with
                   | Some b, l -> Buffer.contents b :: l
                   | None, l -> l
@@ -427,11 +467,11 @@ let command_line =
 
       lookup_dollar (None, []) lst
   in
-    
-    { 
-      parse = 
+
+    {
+      parse =
         (fun ~ctxt s ->
-           match split_expandable s with 
+           match split_expandable s with
              | cmd :: args ->
                  cmd, args
              | [] ->
@@ -439,8 +479,8 @@ let command_line =
       update =
         (fun (cmd, args1) (arg2, args3) ->
            (cmd, args1 @ (arg2 :: args3)));
-      print = 
-        (fun (cmd, args) -> 
+      print =
+        (fun (cmd, args) ->
            space_separated.print (cmd :: args))
     }
 
