@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,14 +20,19 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 open OASISGettext
 
+
 TYPE_CONV_PATH "OASISVersion"
 
+
 type s = string
 
+
 type t = string with odn
 
+
 type comparator =
   | VGreater of t
   | VGreaterEqual of t
@@ -37,18 +43,22 @@ type comparator =
   | VAnd of comparator * comparator
   with odn
 
+
 (* Range of allowed characters *)
 let is_digit c =
   '0' <= c && c <= '9'
 
+
 let is_alpha c =
   ('a' <= c && c <= 'z') || ('A' <= c && c <= 'Z')
 
+
 let is_special =
   function
     | '.' | '+' | '-' | '~' -> true
     | _ -> false
 
+
 let rec version_compare v1 v2 =
   if v1 <> "" || v2 <> "" then
     begin
@@ -92,11 +102,11 @@ let rec version_compare v1 v2 =
             while !p < String.length v && is_digit v.[!p] do
               incr p
             done;
-            let substr = 
+            let substr =
               String.sub v !p ((String.length v) - !p)
-            in 
-            let res = 
-              match String.sub v start_p (!p - start_p) with 
+            in
+            let res =
+              match String.sub v start_p (!p - start_p) with
                 | "" -> 0
                 | s -> int_of_string s
             in
@@ -132,8 +142,14 @@ let rec version_compare v1 v2 =
 
 let version_of_string str = str
 
+
 let string_of_version t = t
 
+
+let version_compare_string s1 s2 =
+  version_compare (version_of_string s1) (version_of_string s2)
+
+
 let chop t =
   try
     let pos =
@@ -143,6 +159,7 @@ let chop t =
   with Not_found ->
     t
 
+
 let rec comparator_apply v op =
   match op with
     | VGreater cv ->
@@ -160,6 +177,7 @@ let rec comparator_apply v op =
     | VAnd (op1, op2) ->
         (comparator_apply v op1) && (comparator_apply v op2)
 
+
 let rec string_of_comparator =
   function
     | VGreater v  -> "> "^(string_of_version v)
@@ -172,6 +190,7 @@ let rec string_of_comparator =
     | VAnd (c1, c2) ->
         (string_of_comparator c1)^" && "^(string_of_comparator c2)
 
+
 let rec varname_of_comparator =
   let concat p v =
     OASISUtils.varname_concat
@@ -190,14 +209,26 @@ let rec varname_of_comparator =
       | VAnd (c1, c2) ->
           (varname_of_comparator c1)^"_and_"^(varname_of_comparator c2)
 
-let version_0_3_or_after t =
-  comparator_apply t (VGreaterEqual (string_of_version "0.3"))
+
+let rec comparator_ge v' =
+  let cmp v = version_compare v v' >= 0 in
+  function
+    | VEqual v
+    | VGreaterEqual v
+    | VGreater v -> cmp v
+    | VLesserEqual _
+    | VLesser _ -> false
+    | VOr (c1, c2) -> comparator_ge v' c1 || comparator_ge v' c2
+    | VAnd (c1, c2) -> comparator_ge v' c1 && comparator_ge v' c2
+
 
 (* END EXPORT *)
 
+
 open OASISUtils
 open OASISVersion_types
 
+
 let comparator_of_string str =
   let lexbuf =
     Lexing.from_string str
@@ -222,6 +253,7 @@ let comparator_of_string str =
         str
         (Printexc.to_string e)
 
+
 let rec comparator_reduce =
   function
     | VAnd (v1, v2) ->
@@ -239,8 +271,10 @@ let rec comparator_reduce =
     | cmp ->
         cmp
 
+
 open OASISValues
 
+
 let value =
   {
     parse  = (fun ~ctxt s -> version_of_string s);
@@ -248,10 +282,10 @@ let value =
     print  = string_of_version;
   }
 
+
 let comparator_value =
   {
     parse  = (fun ~ctxt s -> comparator_of_string s);
     update = update_fail;
     print  = string_of_comparator;
   }
-
