@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,35 +20,40 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 open OASISGettext
 
+
 type name = string
 
+
 exception Not_set of name * string option
 exception No_printer of name
 exception Unknown_field of name * name
 
+
 let () =
   Printexc.register_printer
     (function
        | Not_set (nm, Some rsn) ->
-           Some 
+           Some
              (Printf.sprintf (f_ "Field '%s' is not set: %s") nm rsn)
        | Not_set (nm, None) ->
-           Some 
+           Some
              (Printf.sprintf (f_ "Field '%s' is not set") nm)
        | No_printer nm ->
            Some
              (Printf.sprintf (f_ "No default printer for value %s") nm)
        | Unknown_field (nm, schm) ->
-           Some 
-             (Printf.sprintf (f_ "Field %s is not defined in schema %s") nm schm)
+           Some
+             (Printf.sprintf
+                (f_ "Field %s is not defined in schema %s") nm schm)
        | _ ->
            None)
 
+
 module Data =
 struct
-
   type t =
       (name, unit -> unit) Hashtbl.t
 
@@ -57,10 +63,11 @@ struct
   let clear t =
     Hashtbl.clear t
 
+
 (* END EXPORT *)
-  let elements t = 
+  let elements t =
     let rlst = ref [] in
-      Hashtbl.iter 
+      Hashtbl.iter
         (fun nm _ -> rlst := nm :: !rlst)
         t;
       !rlst
@@ -70,9 +77,9 @@ struct
 (* START EXPORT *)
 end
 
+
 module Schema =
 struct
-
   type ('ctxt, 'extra) value =
       {
         get:   Data.t -> string;
@@ -160,9 +167,9 @@ struct
     t.name
 end
 
+
 module Field =
 struct
-
   type ('ctxt, 'value, 'extra) t =
       {
         set:    Data.t -> ?context:'ctxt -> 'value -> unit;
@@ -292,16 +299,14 @@ struct
 
   let fgets data t =
     t.gets data
-
 end
 
+
 module FieldRO =
 struct
-
   let create ?schema ?name ?parse ?print ?default ?update ?help extra =
     let fld =
       Field.create ?schema ?name ?parse ?print ?default ?update ?help extra
     in
       fun data -> Field.fget data fld
-
 end
