@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,24 +20,28 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (** META generator
     @author Sylvain Le Gall
   *)
 
+
 open OASISGettext
 open OASISTypes
 open OASISValues
-open OASISLibrary
+open OASISFindlib
 open OASISFileTemplate
 open OASISPlugin
 open OASISSchema
 open Format
 
+
 type meta_type =
   | METALibrary
   | METASyntax
 
-type t = 
+
+type t =
     {
       enable:      bool;
       description: string option;
@@ -45,21 +50,32 @@ type t =
       extra_lines: string list;
     }
 
-let plugin = 
+
+let plugin =
   `Extra, "META", Some OASISConf.version_short
 
-let self_id, all_id = 
+
+let self_id, all_id =
   Extra.create plugin
 
-let pivot_data = 
+
+let feature_extra_lines =
+  OASISFeatures.create "extra_lines" ~plugin
+    (OASISFeatures.since_version "0.3")
+    (fun () ->
+       s_ "Allow to add extra lines to the generated META.")
+
+
+let pivot_data =
   data_new_property plugin
 
+
 let generator =
-  let new_field nm = 
+  let new_field nm =
     new_field OASISLibrary.schema all_id nm
   in
 
-  let enable = 
+  let enable =
     new_field
       "Enable"
       ~default:true
@@ -99,7 +115,7 @@ let generator =
     new_field
       "ExtraLines"
       ~default:[]
-      ~since_version:"0.3"
+      ~feature:feature_extra_lines
       (newline_separated string_not_empty)
       (fun () ->
          s_ "Extra lines to add to the META")
@@ -125,7 +141,13 @@ let generator =
         extra_lines = extra_lines data;
       }
 
+
 let pp_print_meta pkg root_t findlib_name_of_library_name fmt grp =
+  let may f =
+    function
+      | Some x -> f x
+      | None -> ()
+  in
 
   let replace_chars s =
     OASISString.replace_chars
@@ -135,101 +157,142 @@ let pp_print_meta pkg root_t findlib_name_of_library_name fmt grp =
       s
   in
 
-  let pp_print_field fmt (var, preds, vl) = 
+  let pp_print_field fmt (var, preds, vl) =
     fprintf fmt
       "@,@[%s(%s) =@ %S@]"
-      var 
+      var
       (String.concat ", " preds)
       (replace_chars vl)
   in
-  let pp_print_sfield fmt (var, vl) = 
+  let pp_print_sfield fmt (var, vl) =
     fprintf fmt "@,@[%s =@ %S@]" var (replace_chars vl)
   in
 
-  let default_synopsis = 
+  let default_synopsis =
     match root_t.description with
       | Some txt -> txt
       | None -> pkg.synopsis
   in
 
-  let rec pp_print_library fmt (lib_cs, lib_bs, lib, children) =
-    let lib_name =
-      lib_cs.cs_name
+  let rec pp_print_library fmt (cs, bs, contents, children) =
+    let name = cs.cs_name in
+    let archive_byte, archive_byte_plugin,
+        archive_native, archive_native_plugin =
+      match contents with
+      | `Library _ ->
+          name^".cma", Some (name^".cma"),
+          name^".cmxa", Some (name^".cmxs")
+      | `Object { obj_modules = [ m ] } ->
+          let dir = OASISHostPath.of_unix bs.bs_path in
+          let exists fn ext =
+            let path = Filename.concat dir (fn ^ ext) in
+            OASISFileUtil.file_exists_case path
+          in
+          let m =
+            prerr_endline (Printf.sprintf "module: %s" m);
+            List.find
+              (fun m -> exists m ".mli" || exists m ".ml")
+              [ String.uncapitalize m ; String.capitalize m ]
+          in
+          m^".cmo", None,
+          m^".cmx", None
+      | `Object _ ->
+          name^".cmo", None,
+          name^".cmx", None
+
     in
-    let lib_cma, lib_cmxa, lib_cmxs =
-      lib_name^".cma", lib_name^".cmxa", lib_name^".cmxs"
+    let t =
+      generator cs.cs_data
     in
-    let t = 
-      generator lib_cs.cs_data
-    in
-      pp_print_sfield fmt ("version", (OASISVersion.string_of_version pkg.version));
+      pp_print_sfield fmt
+        ("version", (OASISVersion.string_of_version pkg.version));
       begin
         let txt =
           match t.description with
             | Some txt -> txt
-            | None -> default_synopsis 
+            | None -> default_synopsis
         in
           pp_print_sfield fmt ("description", txt)
       end;
-      begin 
-        let requires = 
-          match t.requires with 
+      begin
+        let requires =
+          match t.requires with
             | Some lst ->
                 lst
             | None ->
-                List.map 
+                List.map
                   (function
                      | InternalLibrary nm ->
                          findlib_name_of_library_name nm
                      | FindlibPackage (fndlb_nm, _) ->
                          fndlb_nm)
-                  lib_bs.bs_build_depends
+                  bs.bs_build_depends
         in
          if requires <> [] then
           pp_print_sfield fmt ("requires", String.concat " " requires)
-      end; 
+      end;
       begin
-        match t.meta_type with 
+        match t.meta_type with
           | METALibrary ->
-              pp_print_field fmt ("archive", ["byte"], lib_cma);
-              pp_print_field fmt ("archive", ["byte"; "plugin"], lib_cma);
+              pp_print_field fmt ("archive", ["byte"], archive_byte);
+              may
+                (fun x ->
+                   pp_print_field fmt ("archive", ["byte"; "plugin"], x))
+                archive_byte_plugin;
               begin
-                match lib_bs.bs_compiled_object with
-                  | Best | Native ->
-                      pp_print_field fmt ("archive", ["native"], lib_cmxa);
-                      pp_print_field fmt ("archive", ["native"; "plugin"], 
-                                          lib_cmxs)
+                match bs.bs_compiled_object with
+                | Best | Native ->
+                    pp_print_field fmt ("archive", ["native"], archive_native);
+                    may
+                      (fun x ->
+                         pp_print_field fmt
+                           ("archive", ["native"; "plugin"], x))
+                      archive_native_plugin;
                   | Byte ->
                       ()
               end
 
           | METASyntax ->
-              pp_print_field fmt ("archive", ["syntax"; "preprocessor"], lib_cma);
-              pp_print_field fmt ("archive", ["syntax"; "toploop"], lib_cma)
+              pp_print_field fmt
+                ("archive", ["syntax"; "preprocessor"], archive_byte);
+              pp_print_field fmt
+                ("archive", ["syntax"; "toploop"], archive_byte);
+              begin match bs.bs_compiled_object with
+              | Best | Native ->
+                  pp_print_field fmt
+                    ("archive", ["syntax"; "preprocessor"; "native"], archive_native);
+                  may
+                    (fun x ->
+                       pp_print_field fmt
+                         ("archive", ["syntax"; "preprocessor"; "native"; "plugin"], x))
+                    archive_native_plugin;
+              | Byte ->
+                    ()
+              end
       end;
       List.iter (fprintf fmt "@,%s") t.extra_lines;
-      pp_print_sfield fmt 
-        ("exists_if", 
-         if lib_bs.bs_compiled_object = Native then 
-           lib_cmxa
+      pp_print_sfield fmt
+        ("exists_if",
+         if bs.bs_compiled_object = Native then
+           archive_native
          else
-           lib_cma);
+           archive_byte);
       FormatExt.pp_print_list pp_print_group "@," fmt children
 
-  and pp_print_group fmt = 
-    function 
+  and pp_print_group fmt =
+    function
       | Container (fndlb_nm, children) ->
-          fprintf fmt 
+          fprintf fmt
             "@,@[<v1>@[package %S (@]%a%a@]@,)"
             fndlb_nm
-            
-            pp_print_sfield 
+
+            pp_print_sfield
             ("description", "Virtual container")
-            
+
             (FormatExt.pp_print_list pp_print_group "") children
 
       | Package (fndlb_nm, lib_cs, lib_bs, lib, children) ->
-          let t = 
+          let t =
             generator lib_cs.cs_data
           in
             if t.enable then
@@ -242,7 +305,7 @@ let pp_print_meta pkg root_t findlib_name_of_library_name fmt grp =
     pp_open_vbox fmt 0;
     fprintf fmt "# OASIS_START";
     begin
-      match grp with 
+      match grp with
         | Container (_, children) ->
             FormatExt.pp_print_list pp_print_group "" fmt children
         | Package (_, lib_cs, lib_bs, lib, children) ->
@@ -252,14 +315,15 @@ let pp_print_meta pkg root_t findlib_name_of_library_name fmt grp =
     pp_close_box fmt ();
     pp_print_flush fmt ()
 
+
 let main ctxt pkg =
   let group_libs, findlib_name_of_library_name, _ =
     findlib_mapping pkg
   in
   let meta_created = Hashtbl.create 3 in
-    List.fold_left 
+    List.fold_left
       (fun ctxt grp ->
-         let root_cs, root_bs, root_lib = 
+         let root_cs, root_bs, root_lib =
            root_of_group grp
          in
          let root_t =
@@ -295,7 +359,7 @@ let main ctxt pkg =
                       ~template:true
                       meta_fn
                       comment_meta
-                      (OASISUtils.split_newline ~trim:false 
+                      (OASISString.split_newline ~do_trim:false
                          (Buffer.contents buff)))
                    ctxt
              end
