@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,10 +20,12 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (** Generate standard text files: INSTALL.txt, README.txt
     @author Sylvain Le Gall
   *)
 
+
 open OASISFileTemplate
 open OASISPlugin
 open OASISTypes
@@ -35,19 +38,30 @@ open OASISSchema
 open Format
 open FormatExt
 
-let plugin = 
+
+let plugin =
   `Extra, "StdFiles", Some OASISConf.version_short
 
-let self_id, all_id = 
+
+let self_id, all_id =
   Extra.create plugin
 
+
+let markdown =
+  OASISFeatures.create "stdfiles_markdown" ~plugin
+    OASISFeatures.alpha
+    (fun () ->
+       s_ "Use markdown comment and replace .txt extensions of standard files by
+           .md.")
+
 type package =
     (* Standalone executable *)
   | LTool of prog
-    (* Findlib package *)  
+    (* Findlib package *)
   | LFindlibPackage of name * (OASISVersion.comparator option)
 
-let facts = 
+
+let facts =
   [
     LFindlibPackage ("ocaml", None),
     [
@@ -105,55 +119,57 @@ let facts =
       "unix";
     ];
 
-    LFindlibPackage ("findlib", None), 
+    LFindlibPackage ("findlib", None),
     ["ocamlfind"],
     ["findlib"];
   ]
 
+
 (** Merge 2 versions constraint *)
 let merge_version_opt ver_opt1 ver_opt2 =
  match ver_opt1, ver_opt2 with
    | Some v1, Some v2 -> Some (VAnd (v1, v2))
    | None, v | v, None -> v
 
+
 (** Associate a tool to a findlib package or a tool *)
-let package_of_tool = 
-  let mp = 
-    map_string_of_assoc
+let package_of_tool =
+  let mp =
+    MapString.of_list
       (List.fold_left
          (fun acc (pkg, tools, _) ->
             List.fold_left
               (fun acc tool -> (tool, pkg) :: acc)
               acc
               tools)
-         []
-         facts)
+         [] facts)
   in
     fun tool ->
-      try 
+      try
         MapString.find tool mp
       with Not_found ->
         LTool tool
 
+
 (** Associate a library to a findlib package raw data *)
-let package_of_library = 
-  let mp = 
-    map_string_of_assoc
+let package_of_library =
+  let mp =
+    MapString.of_list
       (List.fold_left
          (fun acc (pkg, _, libs) ->
             List.fold_left
               (fun acc lib -> (lib, pkg) :: acc)
               acc
               libs)
-         []
-         facts)
+         [] facts)
   in
     fun lib ver_opt ->
-      try 
+      try
         MapString.find lib mp
       with Not_found ->
         LFindlibPackage (lib, ver_opt)
 
+
 type t =
     {
       readme:  unix_filename option;
@@ -161,33 +177,47 @@ type t =
       authors: unix_filename option;
     }
 
+
 let pivot_data =
   data_new_property plugin
 
-let generator = 
-  let fn_enable fn sync = 
-    let enable = 
-      new_field 
+
+let default_filenames =
+  let mp =
+    MapString.of_list(["README", "README.txt";
+                       "INSTALL", "INSTALL.txt";
+                       "AUTHORS", "AUTHORS.txt"])
+  in
+    fun fn ->
+      try
+        MapString.find fn mp
+      with Not_found ->
+        failwithf (f_ "No default filename for file %s.") fn
+
+let generator =
+  let fn_enable fn sync =
+    let enable =
+      new_field
         OASISPackage.schema
         all_id
-        fn 
-        ~default:true 
+        fn
+        ~default:true
         boolean
         (fun () ->
            Printf.sprintf (f_ "Enable %s file generation.") fn)
-        pivot_data 
+        pivot_data
         (fun t t' -> (sync t t') <> None)
-    in 
-    let fn = 
-      new_field 
+    in
+    let fn =
+      new_field
         OASISPackage.schema
         all_id
         (fn^"Filename")
-        ~default:(fn^".txt")
+        ~default:(default_filenames fn)
         string_not_empty
-        (fun () -> 
+        (fun () ->
            Printf.sprintf (f_ "Real filename to use for file %s.") fn)
-        pivot_data 
+        pivot_data
         (fun t t' ->
            match sync t t' with
              | Some fn -> fn
@@ -204,7 +234,7 @@ let generator =
     fn_enable "README" (fun _ t -> t.readme)
   in
 
-  let install = 
+  let install =
     fn_enable "INSTALL" (fun _ t -> t.install)
   in
 
@@ -212,80 +242,31 @@ let generator =
     fn_enable "AUTHORS" (fun _ t -> t.authors)
   in
 
-    fun data -> 
+    fun data ->
       {
         readme  = readme data;
         install = install data;
-        authors = authors data; 
+        authors = authors data;
       }
 
-module SetSection = 
+
+module SetSection =
   Set.Make
     (struct type t = section let compare = compare end)
 
-let main ctxt pkg = 
-  let data =
-    pkg.schema_data
-  in
 
-  let pp_print_cut2 fmt () =
-    pp_print_cut fmt ();
-    pp_print_cut fmt ()
-  in
-
-  let pp_print_para fmt str = 
-    let str_len =
-      String.length str
-    in
-    let rec decode_string i = 
-      if i < str_len then
-        begin
-          match str.[i] with 
-            | ' ' -> 
-                pp_print_space fmt ();
-                decode_string (i + 1)
-
-            | '\n' -> 
-                if i + 1 < str_len && str.[i + 1] = '\n' then
-                  begin
-                    pp_close_box fmt ();
-                    pp_print_cut2 fmt ();
-                    pp_open_box fmt 0;
-                    decode_string (i + 2)
-                  end
-                else
-                  begin
-                    pp_print_space fmt ();
-                    decode_string (i + 1)
-                  end
-
-            | c -> 
-                pp_print_char fmt c;
-                decode_string (i + 1)
-        end;
-    in
-      pp_open_vbox fmt 0;
-      pp_open_box fmt 0;
-      decode_string 0;
-      pp_close_box fmt ();
-      pp_print_cut2 fmt ();
-      pp_close_box fmt ()
-  in
-
-  let pp_print_title fmt str =
-    (* TODO: use Markdown formatting *)
-    fprintf fmt "@[%s@]@," str;
-    fprintf fmt "@[%s@]@,@," (String.make (String.length str) '=')
-  in
+let main ctxt pkg =
+  let data = pkg.schema_data in
 
   (** All sections that contains a build_section *)
-  let all_build_sections_set = 
+  let all_build_sections_set =
     let all_build_sections =
       List.rev
         (List.fold_left
            (fun acc ->
               function
-                | Library (_, bs, _) 
+                | Library (_, bs, _)
+                | Object (_, bs, _)
                 | Executable (_, bs, _) as sct ->
                     (sct, bs) :: acc
                 | SrcRepo _ | Flag _ | Test _ | Doc _ ->
@@ -295,7 +276,7 @@ let main ctxt pkg =
     in
       List.fold_left
         (fun acc e -> SetSection.add e acc)
-        SetSection.empty  
+        SetSection.empty
         (List.rev_map fst all_build_sections)
   in
 
@@ -305,7 +286,7 @@ let main ctxt pkg =
        is None, otherwise "Some sections".
      *)
     fun fmt sections ->
-      let is_all_build_sections, sections = 
+      let is_all_build_sections, sections =
         if SetSection.subset all_build_sections_set sections then
           true, SetSection.diff sections all_build_sections_set
         else
@@ -318,7 +299,7 @@ let main ctxt pkg =
             (fun fmt ->
                if is_all_build_sections then
                  fprintf fmt "all,@ ")
-            (pp_print_list 
+            (pp_print_list
                (fun fmt sct ->
                   fprintf fmt "%s" (OASISSection.string_of_section sct))
                ",@ ")
@@ -326,46 +307,44 @@ let main ctxt pkg =
   in
 
   let pp_print_ver_opt fmt =
-    function
-      | Some ver_cmp ->
-          fprintf fmt " (%a)"
-            pp_print_string 
-            (OASISVersion.string_of_comparator 
-               (OASISVersion.comparator_reduce ver_cmp))
-      | None ->
-          ()
+    may
+      (fun ver_cmp ->
+         fprintf fmt " (%a)"
+           pp_print_string
+           (OASISVersion.string_of_comparator
+              (OASISVersion.comparator_reduce ver_cmp)))
   in
 
-  let depends = 
-    let merge_package_section lst (new_pkg, new_sections) = 
+  let depends =
+    let merge_package_section lst (new_pkg, new_sections) =
       try
-        let old_pkg, new_pkg, old_sections = 
-          match new_pkg with 
+        let old_pkg, new_pkg, old_sections =
+          match new_pkg with
             | LTool _ as tool ->
-                tool, 
+                tool,
                 tool,
                 List.assoc tool lst
-                          
+
             | LFindlibPackage (nm1, ver_opt1) ->
                 begin
-                  let res_opt = 
+                  let res_opt =
                     List.fold_left
                       (fun acc pkg ->
-                         match acc, pkg with 
-                           | None, 
-                             ((LFindlibPackage (nm2, ver_opt2)) 
-                                as old_pkg, 
-                              sections) -> 
+                         match acc, pkg with
+                           | None,
+                             ((LFindlibPackage (nm2, ver_opt2))
+                                as old_pkg,
+                              sections) ->
                                if nm1 = nm2 then
-                                 Some 
+                                 Some
                                    (old_pkg,
-                                    LFindlibPackage 
-                                      (nm1, 
-                                       merge_version_opt 
-                                         ver_opt1 
+                                    LFindlibPackage
+                                      (nm1,
+                                       merge_version_opt
+                                         ver_opt1
                                          ver_opt2),
                                     sections)
-                               else 
+                               else
                                  acc
                            | _, _ ->
                                acc)
@@ -379,8 +358,8 @@ let main ctxt pkg =
                           raise Not_found
                 end
         in
-        let lst = 
-          try 
+        let lst =
+          try
             List.remove_assoc old_pkg lst
           with Not_found ->
             lst
@@ -391,7 +370,7 @@ let main ctxt pkg =
         (new_pkg, new_sections) :: lst
     in
 
-    let add_build_tools ssection = 
+    let add_build_tools ssection =
        List.fold_left
          (fun lst ->
             function
@@ -401,35 +380,37 @@ let main ctxt pkg =
                   lst)
     in
 
-    let split_package_section = 
-      List.fold_left 
+    let split_package_section =
+      List.fold_left
         (fun lst ->
-           function 
-             | Library (_, bs, _) | Executable (_, bs, _) as section ->
+           function
+             | Library (_, bs, _)
+             | Object (_, bs, _)
+             | Executable (_, bs, _) as section ->
                  begin
-                   let ssection = 
+                   let ssection =
                      SetSection.singleton section
                    in
-                   let lst = 
+                   let lst =
                      (* Add build_depends *)
                      List.fold_left
                        (fun lst ->
                           function
                             | FindlibPackage (fndlb_nm, ver_opt) ->
-                                let fndlb_root = 
+                                let fndlb_root =
                                   match (OASISString.nsplit fndlb_nm '.') with
                                     | hd :: _ -> hd
                                     | _ -> fndlb_nm
                                 in
                                   (package_of_library fndlb_root ver_opt,
-                                   ssection) :: lst 
+                                   ssection) :: lst
                             | InternalLibrary _ ->
                                 lst)
                        lst
                        bs.bs_build_depends
                    in
                      (* Add build_tools *)
-                     add_build_tools 
+                     add_build_tools
                        ssection
                        lst
                        bs.bs_build_tools
@@ -441,13 +422,27 @@ let main ctxt pkg =
                    lst
                    build_tools
              | Flag _ | SrcRepo _ ->
-                 lst) 
+                 lst)
 
         (* Basic dependencies *)
-        [
-          LFindlibPackage ("findlib", pkg.findlib_version), all_build_sections_set;
-          LFindlibPackage ("ocaml", pkg.ocaml_version), all_build_sections_set;
-        ]
+        ([
+          LFindlibPackage ("findlib", pkg.findlib_version),
+          all_build_sections_set;
+
+          LFindlibPackage ("ocaml", pkg.ocaml_version),
+          all_build_sections_set]
+        @
+         (if OASISFeatures.package_test
+               OASISFeatures.dynrun_for_release pkg then
+            [
+              LFindlibPackage ("oasis",
+                               Some (OASISVersion.VGreaterEqual
+                                       pkg.oasis_version)),
+              all_build_sections_set
+            ]
+          else
+            []))
+
 
         (* Go through all sections *)
         pkg.sections
@@ -460,21 +455,44 @@ let main ctxt pkg =
         split_package_section
   in
 
-  let add_file ctxt (fn_opt, ppf) =
-    match fn_opt with 
+  let add_file ctxt (fn_opt, important, ppf) =
+    match fn_opt with
       | Some unix_fn ->
           begin
-            let content = 
+            let rec remove_eol_eof str =
+              List.fold_left
+                (fun str eol ->
+                   try
+                     remove_eol_eof (OASISString.strip_ends_with ~what:eol str)
+                   with Not_found ->
+                     str)
+                str ["\n"; "\r"]
+            in
+            let content =
               ppf str_formatter;
-              flush_str_formatter ()
+              (* Trim problematic 2x newline at end of last para, because we
+               * don't know if there will a following para.
+               *)
+              remove_eol_eof (flush_str_formatter ())
+            in
+            let comment =
+              if OASISFeatures.package_test markdown pkg then
+                comment_markdown
+              else
+                comment_ml
             in
               add_file
-                (template_make
-                   unix_fn
-                   comment_ml
-                   []
-                   [content]
-                   [])
+                {(template_make unix_fn comment
+                    []
+                    [
+                      (* One line before, one line after to avoid mixing content
+                       * with comment formatting (it causes e.g. pandoc to think
+                       * this is a paragraph.
+                       *)
+                      "\n"^content^"\n"
+                    ]
+                    []) with
+                     important = important}
                 ctxt
           end
 
@@ -482,8 +500,25 @@ let main ctxt pkg =
           ctxt
   in
 
-  let t = 
-    generator data 
+  let t = generator data in
+
+  let fix_ext fn real_fn_opt =
+    match real_fn_opt with
+      | Some real_fn ->
+          if OASISFeatures.package_test markdown pkg &&
+             real_fn = default_filenames fn then
+            Some (fn^".md")
+          else
+            Some real_fn
+      | None -> None
+  in
+
+  let t =
+    {
+      readme = fix_ext "README" t.readme;
+      install = fix_ext "INSTALL" t.install;
+      authors = fix_ext "AUTHORS" t.authors;
+    }
   in
 
     List.fold_left
@@ -492,126 +527,120 @@ let main ctxt pkg =
       [
         (* Generate README.txt *)
         t.readme,
+        false,
         (fun fmt ->
            pp_open_vbox fmt 0;
-           fprintf fmt 
-             "@[This is the README file for the %s distribution.@]@,@,"
-             pkg.name;
+           pp_print_titlef fmt 1 "%s - %s" pkg.name pkg.synopsis;
+
+           may
+             (fun descr ->
+                OASISText.pp_print fmt descr;
+                pp_print_newline fmt ();
+                pp_print_newline fmt ())
+             pkg.description;
+
+           may
+             (fun fn ->
+                pp_print_paraf fmt
+                  "See the file [%s](%s) for building and installation \
+                   instructions." fn fn)
+             t.install;
+
+           may (pp_print_paraf fmt "[Home page](%s)") pkg.homepage;
+
+           pp_print_title fmt 2 "Copyright and license";
 
            List.iter
-             (pp_print_para fmt)
+             (fun str ->
+                pp_print_string fmt str;
+                pp_print_newline fmt ())
              pkg.copyrights;
-           
-           pp_print_para fmt pkg.synopsis;
-           
-           begin
-             match pkg.description with 
-               | Some str ->
-                   pp_print_para fmt str
-               | None ->
-                   ()
-           end;
+           if pkg.copyrights <> [] then
+             pp_print_newline fmt ();
+
+           pp_print_para fmt
+             (OASISLicense.legal_disclaimer pkg.name pkg.license);
+
+           may
+             (fun fn ->
+                pp_print_paraf fmt "See [%s](%s) for more information." fn fn)
+             pkg.license_file;
 
-           pp_open_box fmt 0;
-           begin
-             match t.install with 
-               | Some fn -> 
-                   fprintf fmt
-                     "See@ the@ files@ %s@ for@ building@ and@ installation@ instructions.@ "
-                     fn
-               | None ->
-                   ()
-           end;
-           begin
-             match pkg.license_file with 
-               | Some fn ->
-                   fprintf fmt "See@ the@ file@ %s@ for@ copying@ conditions.@ " fn
-               | None ->
-                   ()
-           end;
            pp_close_box fmt ();
-           pp_print_cut2 fmt ();
+        );
 
-           begin
-             match pkg.homepage with 
-               | Some url ->
-                   fprintf fmt
-                     "@[<hv2>Home page: %s@]@,@," url;
-               | None -> 
-                   ()
-           end;
+        (* Generate INSTALL.txt *)
+        t.install,
+        (OASISFeatures.package_test OASISFeatures.dynrun_for_release pkg),
+        (fun fmt ->
+           pp_open_vbox fmt 0;
+           fprintf fmt
+             "@[This is the INSTALL file for the %s distribution.@]@,@,"
+             pkg.name;
 
+           pp_print_para fmt
+             "This package uses OASIS to generate its build system. \
+              See section OASIS for full information.";
+
+           pp_print_title fmt 1 "Dependencies";
+           fprintf fmt "@[In order to compile this package, you will need:@]@,";
+           pp_open_vbox fmt 0;
+           pp_print_cut fmt ();
+           pp_print_list
+             (fun fmt (pkg, sections) ->
+                fprintf fmt "* @[%a%a@]"
+                  (fun fmt ->
+                     function
+                       | LTool s ->
+                           pp_print_string fmt s
+                       | LFindlibPackage (nm, ver_opt) ->
+                           fprintf fmt "%s%a"
+                             nm
+                             pp_print_ver_opt ver_opt)
+                  pkg
+                  pp_print_sections sections)
+             "@,"
+             fmt
+             depends;
+
+           pp_close_box fmt ();
+           pp_print_cut fmt ();
+
+           pp_print_string fmt StdFilesData.install;
            pp_close_box fmt ());
 
-      (* Generate INSTALL.txt *)
-      t.install,
-      (fun fmt ->
-         pp_open_vbox fmt 0;
-         fprintf fmt 
-           "@[This is the INSTALL file for the %s distribution.@]@,@,"
-           pkg.name;
+        (* Generate AUTHORS.txt *)
+        t.authors,
+        false,
+        (fun fmt ->
+           pp_open_vbox fmt 0;
+           fprintf fmt "@[Authors of %s:@]@," pkg.name;
+           pp_print_cut fmt ();
+           pp_print_list
+             (fun fmt str -> pp_print_string fmt ("* "^str))
+             "@,"
+             fmt
+             pkg.OASISTypes.authors;
 
-         pp_print_para fmt
-           "This package uses OASIS to generate its build system. \
-            See section OASIS for full information. ";
-
-         pp_print_title fmt "Dependencies";
-         fprintf fmt "@[In order to compile this package, you will need:@]@,";
-         pp_open_vbox fmt 0;
-
-         pp_print_list 
-           (fun fmt (pkg, sections) ->
-              fprintf fmt "* @[%a%a@]" 
-                (fun fmt ->
-                   function
-                     | LTool s -> 
-                         pp_print_string fmt s
-                     | LFindlibPackage (nm, ver_opt) ->
-                         fprintf fmt "%s%a"
-                           nm
-                           pp_print_ver_opt ver_opt)
-                pkg
-                pp_print_sections sections)
-           "@,"
-           fmt
-           depends;
-
-         pp_close_box fmt ();
-         pp_print_cut fmt ();
-
-         pp_print_string fmt StdFilesData.install;
-         pp_close_box fmt ());
-      
-      (* Generate AUTHORS.txt *)
-      t.authors,
-      (fun fmt ->
-         pp_open_vbox fmt 0;
-         fprintf fmt "@[Authors of %s@]@," pkg.name;
-
-         pp_print_list
-           pp_print_string
-           "@,"
-           fmt
-           pkg.OASISTypes.authors;
-
-         if pkg.maintainers <> [] then
-           begin
+           if pkg.maintainers <> [] then begin
              pp_print_cut2 fmt ();
-             fprintf fmt "@[Current maintainers of %s@]@," pkg.name;
+             fprintf fmt "@[Current maintainers of %s:@]@," pkg.name;
+             pp_print_cut fmt ();
              pp_print_list
-               pp_print_string
+               (fun fmt str -> pp_print_string fmt ("* "^str))
                "@,"
                fmt
                pkg.maintainers
            end;
 
-         pp_close_box fmt ());
+           pp_close_box fmt ());
       ]
 
-let init () = 
+
+let init () =
   register_help plugin
-    {(help_default StdFilesData.readme_template_mkd) with 
+    {(help_default StdFilesData.readme_template_mkd) with
          help_order = 50};
   Extra.register_act self_id main;
-  register_generator_package all_id pivot_data generator 
+  register_generator_package all_id pivot_data generator
 
