@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,34 +20,26 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (** Configure using internal scheme
     @author Sylvain Le Gall
   *)
 
+
 open BaseEnv
 open OASISTypes
 open OASISUtils
 open OASISGettext
 open BaseMessage
 
+
 (** Configure build using provided series of check to be done
   * and then output corresponding file.
   *)
 let configure pkg argv =
-  let var_ignore_eval var =
-    let _s : string =
-      var ()
-    in
-      ()
-  in
-
-  let errors =
-    ref SetString.empty
-  in
-
-  let buff =
-    Buffer.create 13
-  in
+  let var_ignore_eval var = let _s: string = var () in () in
+  let errors = ref SetString.empty in
+  let buff = Buffer.create 13 in
 
   let add_errors fmt =
     Printf.kbprintf
@@ -194,6 +187,20 @@ let configure pkg argv =
       | None ->
           ()
   end;
+  (* Make sure the findlib version is fine for the OCaml compiler. *)
+  begin
+    let ocaml_ge4 =
+      OASISVersion.version_compare
+        (OASISVersion.version_of_string (BaseStandardVar.ocaml_version()))
+        (OASISVersion.version_of_string "4.0.0") >= 0 in
+    if ocaml_ge4 then
+      let findlib_lt132 =
+        OASISVersion.version_compare
+          (OASISVersion.version_of_string (BaseStandardVar.findlib_version()))
+          (OASISVersion.version_of_string "1.3.2") < 0 in
+      if findlib_lt132 then
+        add_errors "OCaml >= 4.0.0 requires Findlib version >= 1.3.2"
+  end;
 
   (* FlexDLL *)
   if BaseStandardVar.os_type () = "Win32" ||
@@ -256,14 +263,18 @@ let configure pkg argv =
         (SetString.cardinal !errors)
     end
 
+
 (* END EXPORT *)
 
+
 open OASISPlugin
 open InternalId
 
+
 let plugin =
   `Configure, name, Some version
 
+
 let init () =
   let self_id, _ =
     Configure.create plugin
