@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,23 +20,33 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (* Create documentation using ocamlbuild .odocl files
    @author Sylvain Le Gall
  *)
 
+
 open OASISTypes
 open OASISGettext
 open OASISMessage
 open OCamlbuildCommon
 open BaseStandardVar
 
+
 TYPE_CONV_PATH "OCamlbuildDocPlugin"
 
-let doc_build path pkg (cs, doc) argv =
+type run_t =
+  {
+    extra_args: string list;
+    run_path: unix_filename;
+  } with odn
+
+
+let doc_build run pkg (cs, doc) argv =
   let index_html =
     OASISUnixPath.make
       [
-        path;
+        run.run_path;
         cs.cs_name^".docdir";
         "index.html";
       ]
@@ -44,11 +55,11 @@ let doc_build path pkg (cs, doc) argv =
     OASISHostPath.make
       [
         build_dir argv;
-        OASISHostPath.of_unix path;
+        OASISHostPath.of_unix run.run_path;
         cs.cs_name^".docdir";
       ]
   in
-    run_ocamlbuild [index_html] argv;
+    run_ocamlbuild (index_html :: run.extra_args) argv;
     List.iter
       (fun glb ->
          BaseBuilt.register
@@ -58,12 +69,15 @@ let doc_build path pkg (cs, doc) argv =
               (Filename.concat tgt_dir glb)])
       ["*.html"; "*.css"]
 
-let doc_clean t pkg (cs, doc) argv =
+
+let doc_clean run pkg (cs, doc) argv =
   run_clean argv;
   BaseBuilt.unregister BaseBuilt.BDoc cs.cs_name
 
+
 (* END EXPORT *)
 
+
 open OASISFileTemplate
 open OASISPlugin
 open OASISValues
@@ -72,9 +86,11 @@ open OASISPlugin
 open OASISSchema
 open OCamlbuildId
 
+
 let plugin =
   `Doc, name, Some version
 
+
 type t =
     {
       path:      unix_dirname;
@@ -82,14 +98,18 @@ type t =
       libraries: findlib_full list;
       intro:     unix_filename option;
       flags:     string list;
+      common:    ocamlbuild_common;
     }
 
-let pivot_data =
-  data_new_property plugin
+
+let pivot_data = data_new_property plugin
+let pivot_sub_data = data_new_property plugin
+
 
 let self_id, all_id =
   Doc.create plugin
 
+
 let new_field nm ?default vl hlp sync =
   new_field
     OASISDocument.schema
@@ -100,6 +120,7 @@ let new_field nm ?default vl hlp sync =
     (fun () -> s_ hlp)
     pivot_data sync
 
+
 let path =
   new_field
     "Path"
@@ -107,6 +128,7 @@ let path =
     (ns_ "Top level directory for building ocamldoc documentation")
     (fun _ t -> t.path)
 
+
 let modules =
   new_field
     "Modules"
@@ -115,14 +137,17 @@ let modules =
     (ns_ "List of OCaml modules used to generate ocamldoc documentation")
     (fun _ t -> t.modules)
 
+
 let libraries =
   new_field
     "Libraries"
     ~default:[]
     (comma_separated findlib_full)
-    (ns_ "Findlib names of internal libraries used to generate the ocamldoc documentation")
+    (ns_ "Findlib names of internal libraries used to generate the ocamldoc \
+          documentation")
     (fun _ t -> t.libraries)
 
+
 (* TODO: the following 2 options require to edit _tags after OCamlbuildDoc
  *)
 (*
@@ -131,9 +156,11 @@ let intro =
     ~default:None
     "Intro"
     (opt file)
-    (ns_ "OCamldoc formatted file used to generate index.html of the ocamldoc documentation")
+    (ns_ "OCamldoc formatted file used to generate index.html of the ocamldoc \
+          documentation")
     (fun _ t -> t.intro)
 
+
 let flags =
   new_field
     ~default:[]
@@ -143,77 +170,93 @@ let flags =
     (fun _ t -> t.flags)
  *)
 
+
 (* TODO: use -t for title *)
 
-let doit ctxt pkg (cs, doc) =
 
-  let path =
-    path cs.cs_data
+let generator =
+  let generator_common =
+    (* Register fields. *)
+    ocamlbuild_common_generator pivot_sub_data OASISDocument.schema all_id
   in
+    fun data pkg ->
+      let path = path data in
 
-  let modules_from_libraries =
-    (* Convert findlib name to internal library and compute
-     * the module they shipped.
-     *)
-    let lib_of_findlib =
-      let _, _, library_name_of_findlib_name =
-        OASISLibrary.findlib_mapping pkg
-      in
-      let lib_of_name =
-        List.fold_left
-          (fun mp ->
-             function
-               | Library ({cs_name = name}, bs, lib) ->
-                   MapString.add name (bs, lib) mp
-               | _ ->
-                   mp)
-          MapString.empty
-          pkg.sections
-      in
-        fun fndlb_nm ->
-          let nm =
-            library_name_of_findlib_name fndlb_nm
+      let modules_from_libraries =
+        (* Convert findlib name to internal library and compute
+         * the module they shipped.
+         *)
+        let lib_of_findlib =
+          let _, _, library_name_of_findlib_name =
+            OASISFindlib.findlib_mapping pkg
           in
-            MapString.find nm lib_of_name
-    in
-
-      (* Fetch modules from internal libraries *)
-      List.flatten
-        (List.map
-           (fun fndlb_nm ->
-              let bs, lib =
-                lib_of_findlib fndlb_nm
+          let lib_of_name =
+            List.fold_left
+              (fun mp ->
+                 function
+                   | Library ({cs_name = name}, bs, lib) ->
+                       MapString.add name (bs, lib) mp
+                   | _ ->
+                       mp)
+              MapString.empty
+              pkg.sections
+          in
+            fun fndlb_nm ->
+              let nm =
+                library_name_of_findlib_name fndlb_nm
               in
-                (* Rebase modules in the doc path *)
-                List.map
-                  (fun modul ->
-                     OASISUnixPath.make_relative
-                       path
-                       (OASISUnixPath.concat bs.bs_path modul))
-                  lib.lib_modules)
+                MapString.find nm lib_of_name
+        in
 
-           (libraries cs.cs_data))
-  in
+          (* Fetch modules from internal libraries *)
+          List.flatten
+            (List.map
+               (fun fndlb_nm ->
+                  let bs, lib =
+                    lib_of_findlib fndlb_nm
+                  in
+                    (* Rebase modules in the doc path *)
+                    List.map
+                      (fun modul ->
+                         OASISUnixPath.make_relative
+                           path
+                           (OASISUnixPath.concat bs.bs_path modul))
+                      lib.lib_modules)
 
-  let modules_from_doc =
-    (* Fetch modules defined directly *)
-    modules cs.cs_data
-  in
+               (libraries data))
+      in
 
-  let modules =
-    modules_from_libraries @ modules_from_doc
-  in
+      let modules_from_doc =
+        (* Fetch modules defined directly *)
+        modules data
+      in
+
+      let modules =
+        modules_from_libraries @ modules_from_doc
+      in
+      {
+        path = path;
+        modules = modules;
+        libraries = libraries data;
+        intro = None;
+        flags = [];
+        common = generator_common data;
+      }
+
+
+let doit ctxt pkg (cs, doc) =
+  let t = generator cs.cs_data pkg in
 
   let ctxt =
     (* Create .odocl file *)
     add_file
       (template_make
          (OASISHostPath.add_extension
-            (Filename.concat path cs.cs_name)
+            (Filename.concat t.path cs.cs_name)
             "odocl")
          comment_ocamlbuild
          []
-         modules
+         t.modules
          [])
       ctxt
   in
@@ -238,32 +281,39 @@ let doit ctxt pkg (cs, doc) =
              cs.cs_name);
 
         set_error
-          (modules = [])
+          (t.modules = [])
           (Printf.sprintf
              (f_ "No module defined for document %s.")
              cs.cs_name);
       ]
   in
 
-    ctxt,
+  let run =
     {
-      chng_moduls =
-        [OCamlbuildData.ocamlbuildsys_ml];
-
-      chng_main =
-        ODNFunc.func_with_arg
-          doc_build "OCamlbuildDocPlugin.doc_build"
-          path ODN.of_string;
-
-      chng_clean =
-        Some
-          (ODNFunc.func_with_arg
-             doc_clean "OCamlbuildDocPlugin.doc_clean"
-             path ODN.of_string);
-
-      chng_distclean =
-        None;
+      run_path = t.path;
+      extra_args = extra_args_ocamlbuild_common ~ctxt:ctxt.ctxt pkg t.common;
     }
+  in
+  ctxt,
+  {
+    chng_moduls =
+      [OCamlbuildData.ocamlbuildsys_ml];
+
+    chng_main =
+      ODNFunc.func_with_arg
+        doc_build "OCamlbuildDocPlugin.doc_build"
+        run odn_of_run_t;
+
+    chng_clean =
+      Some
+        (ODNFunc.func_with_arg
+           doc_clean "OCamlbuildDocPlugin.doc_clean"
+           run odn_of_run_t);
+
+    chng_distclean =
+      None;
+  }
+
 
 let qstrt_completion pkg =
   List.fold_left
@@ -272,6 +322,7 @@ let qstrt_completion pkg =
     [ExternalTool "ocamlbuild";
      ExternalTool "ocamldoc"]
 
+
 let init () =
   OCamlbuildId.init ();
   Doc.register_act self_id doit;
