@@ -312,7 +312,7 @@ let bs_tags pkg sct cs bs src_dirs src_internal_dirs link_tgt ctxt tag_t
     List.rev_append
       (* .ml files *)
       (List.rev_map
-         (fun dn -> Tag.filename_concat dn "*.ml{,i}")
+         (fun dn -> Tag.filename_concat dn "*.ml{,i,y}")
          (src_dirs @ src_internal_dirs))
       (* .c files *)
       (List.map
@@ -1085,6 +1085,8 @@ let add_ocamlbuild_files ctxt pkg =
     ::
     "# useless stuff for the build process"
     ::
+    "true: annot, bin_annot"
+    ::
     add_tags
       tag_t
       ["_darcs"; ".git"; ".hg"; ".bzr"; "**/.svn"]
@@ -1122,8 +1124,12 @@ let add_ocamlbuild_files ctxt pkg =
               Format.flush_str_formatter ()
             );
             "";
+            Printf.sprintf
+              "let conf = {MyOCamlbuildFindlib.no_automatic_syntax = %b}"
+              (OASISFeatures.package_test OASISFeatures.no_automatic_syntax pkg);
+            "";
             "let dispatch_default = \
-                   MyOCamlbuildBase.dispatch_default package_default;;";
+                   MyOCamlbuildBase.dispatch_default conf package_default;;";
             "";
           ]
           ["Ocamlbuild_plugin.dispatch dispatch_default;;"];
