@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,22 +20,32 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (* Work in progress, only for testing purpose
    Try to guess file to install depending on archive (.cma) and published
    interface (.cmi)
  *)
 
-#use "topfind";;
-#require "unix";;
-#require "pcre";;
 
-let warning msg = 
+let () =
+  try Topdirs.dir_directory (Sys.getenv "OCAML_TOPLEVEL_PATH")
+  with Not_found -> ()
+
+
+#use "topfind"
+#require "unix"
+#require "pcre"
+
+
+let warning msg =
   prerr_endline msg
 
-let error msg = 
+
+let error msg =
   prerr_endline msg;
   exit 1
 
+
 let find_unit directory unit_name extensions =
   let unit_base_fn =
     [
@@ -42,11 +53,11 @@ let find_unit directory unit_name extensions =
       (String.uncapitalize unit_name);
     ]
   in
-  let unit_ext_fn = 
+  let unit_ext_fn =
     List.flatten
-      (List.rev_map 
-         (fun unit_nm -> 
-            List.rev_map 
+      (List.rev_map
+         (fun unit_nm ->
+            List.rev_map
               (fun ext -> unit_nm^"."^ext)
               extensions
          )
@@ -54,13 +65,13 @@ let find_unit directory unit_name extensions =
       )
   in
   let unit_fn =
-    List.rev_map 
-      (fun fn -> 
+    List.rev_map
+      (fun fn ->
          Filename.concat directory fn
       )
       unit_ext_fn
   in
-  let () = 
+  let () =
     prerr_endline (String.concat " -> " unit_fn)
   in
   let unit_exist_fn =
@@ -68,37 +79,38 @@ let find_unit directory unit_name extensions =
       Sys.file_exists
       unit_fn
   in
-    match unit_exist_fn with 
+    match unit_exist_fn with
       | fn :: _ ->
           fn
       | [] ->
           raise Not_found
 
+
 let cmx_of_cma cma =
   let dirname =
     Filename.dirname cma
   in
-  let () = 
+  let () =
     if not (Filename.check_suffix cma ".cma") then
       failwith (cma^" is not an OCaml library")
   in
   let unit_name =
-    let reg = 
+    let reg =
       Pcre.regexp "Unit name: ([A-Z][A-Za-z0-9]*)"
     in
       fun line ->
         Pcre.get_substring (Pcre.exec ~rex:reg line) 1
   in
-  let unit_list = 
+  let unit_list =
     ref []
   in
   let chn =
     Unix.open_process_in ("ocamlobjinfo "^cma)
   in
-  let () = 
-    try 
+  let () =
+    try
       while true do
-        try 
+        try
           unit_list := (unit_name (input_line chn)) :: !unit_list
         with Not_found ->
           ()
@@ -109,7 +121,7 @@ let cmx_of_cma cma =
   let _ =
     Unix.close_process_in chn
   in
-  let add_cmx acc unit_nm = 
+  let add_cmx acc unit_nm =
     try
       (find_unit dirname unit_nm ["cmx"]) :: acc
     with Not_found ->
@@ -120,20 +132,21 @@ let cmx_of_cma cma =
   in
     List.fold_left add_cmx [] !unit_list
 
+
 let install_lib ~directory ~archive ~interfaces =
   let mandatory_file fn =
     if not (Sys.file_exists fn) then
       error ("Could not find file '"^fn^"'");
     fn
   in
-  let fn_cma = 
+  let fn_cma =
     mandatory_file (Filename.concat directory (archive^".cma"))
   in
-  let fn_a = 
+  let fn_a =
     mandatory_file (Filename.concat directory (archive^".a"))
   in
-  let fn_lst_cmi = 
-    List.rev_map 
+  let fn_lst_cmi =
+    List.rev_map
       (fun interf ->
          try
            find_unit directory interf ["cmi"]
@@ -160,9 +173,9 @@ let install_lib ~directory ~archive ~interfaces =
   let fn_lst_stubs =
     (* TODO *)
     List.flatten
-      (List.rev_map 
+      (List.rev_map
          (fun archive ->
-            let fn_stubs_a = 
+            let fn_stubs_a =
               (* FIXME msvc: .lib .dll ? *)
               mandatory_file
                 (Filename.concat directory ("lib"^archive^"_stubs.a"))
@@ -177,7 +190,7 @@ let install_lib ~directory ~archive ~interfaces =
       )
   in
   let fn_lst_native =
-    let fn_cmxa = 
+    let fn_cmxa =
       Filename.concat directory (archive^".cmxa")
     in
       if Sys.file_exists fn_cmxa then
@@ -192,7 +205,7 @@ let install_lib ~directory ~archive ~interfaces =
           []
         )
   in
-    List.flatten 
+    List.flatten
       [
         [fn_cma; fn_a];
         fn_lst_cmi;
@@ -201,11 +214,14 @@ let install_lib ~directory ~archive ~interfaces =
         fn_lst_stubs;
       ]
 
+
 (* QUID: .o (.obj), .a (.lib), .so (.dll) *)
 
-let () = 
-  print_endline 
+
+let () =
+  print_endline
     ("To install: "^
-     (String.concat ", " 
-        (install_lib ~directory:"." ~archive:"stlang" ~interfaces:["STLang"; "STLangTypes"])))
+     (String.concat ", "
+        (install_lib ~directory:"." ~archive:"stlang"
+           ~interfaces:["STLang"; "STLangTypes"])))
 
