@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,56 +20,65 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
-open OUnit;;
-open TestCommon;;
+
+open OUnit2
+open TestCommon
+
 
 let tests =
   "Basic" >:::
   [
     "Help1" >::
-    (fun () -> assert_oasis_cli ["--help"]);
- 
+    (fun test_ctxt -> assert_oasis_cli ~ctxt:test_ctxt ["--help"]);
+
     "Help2" >::
-    (fun () -> assert_oasis_cli ["help"]);
- 
+    (fun test_ctxt -> assert_oasis_cli ~ctxt:test_ctxt ["help"]);
+
     "Env dump/load" >::
-    (fun () ->
+    (fun test_ctxt ->
+       (* TODO: lock *)
        BaseEnv.unload ();
-       BaseEnv.load ~filename:(in_data "dir.data") ();
+       BaseEnv.load ~filename:(in_testdata_dir test_ctxt
+                                 ["TestBasic"; "dir.data"]) ();
        (* Reset lazy values ? *)
-       assert_equal 
+       assert_equal
          ~printer:(fun s -> s)
          "toto"
          (BaseStandardVar.bindir ()));
 
     "OCaml dev version" >::
-    (fun () ->
+    (fun test_ctxt ->
        PropList.Schema.set
-         BaseEnv.schema 
+         BaseEnv.schema
          BaseEnv.env
          ~context:BaseEnv.OCommandLine
          "ocamlc_config_map"
-         (Marshal.to_string 
-            (MapString.add 
+         (Marshal.to_string
+            (MapString.add
                "version"
-               "3.13.0+dev2 (2010-10-22)" 
+               "3.13.0+dev2 (2010-10-22)"
                MapString.empty)
             []);
-       assert_equal 
+       assert_equal
          ~printer:(fun s -> s)
          "3.13.0"
          (BaseStandardVar.ocaml_version ()));
 
     "Compileable BaseSysBundle.ml" >::
-    TestDevFiles.bracket_tmpdir
-      (fun dn ->
-         let chn = open_out (Filename.concat dn "bundle.ml") in
-           output_string chn OASISData.oasissysbundle_ml;
-           output_string chn BaseData.basesysbundle_ml;
-           close_out chn;
-           assert_command
-             "ocamlc" ["-o"; (Filename.concat dn "bundle");
-                       (Filename.concat dn "bundle.ml")])
+    (fun test_ctxt ->
+       let dn = bracket_tmpdir test_ctxt in
+       let chn = open_out (Filename.concat dn "bundle.ml") in
+         output_string chn OASISData.oasissysbundle_ml;
+         output_string chn BaseData.basesysbundle_ml;
+         close_out chn;
+         assert_command ~ctxt:test_ctxt
+           "ocamlc" ["-o"; (Filename.concat dn "bundle");
+                     (Filename.concat dn "bundle.ml")]);
+
+    "Sync OASISParse/BaseSetup.default_oasis_fn" >::
+    (fun test_ctxt ->
+       assert_equal
+         ~printer:(fun s -> s)
+         OASISParse.default_oasis_fn
+         BaseSetup.default_oasis_fn);
   ]
-;;
-   
