@@ -584,6 +584,12 @@ let register_installed_files test_ctxt t installed_files_lst =
                acc
            | "cmxs" when not t.native_dynlink ->
                acc
+           | "annot" when
+               OASISVersion.version_compare_string t.ocaml_version "3.11" < 0 ->
+               acc
+           | "cmt" | "cmti" when
+               OASISVersion.version_compare_string t.ocaml_version "4.00" < 0 ->
+               acc
            | "a" ->
                let fn =
                  if is_win32 then FilePath.replace_extension fn "lib" else fn
