@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,16 +20,19 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (** Test for OASISLicense
     @author Sylvain Le Gall
   *)
 
-open OUnit
+
+open OUnit2
 open TestCommon
 open OASISValues
 open OASISTypes
 open OASISLicense
 
+
 let tests =
   let ver =
     OASISVersion.version_of_string
@@ -49,17 +53,17 @@ let tests =
     (List.map
        (fun (txt, res) ->
           txt >::
-          (fun () ->
+          (fun test_ctxt ->
              match res with
                | Some exp ->
-                   assert_equal 
+                   assert_equal
                      ~printer:(fun v -> Printf.sprintf "%S" (to_string v))
                      exp
-                     (value.parse ~ctxt:!oasis_ctxt txt)
+                     (value.parse ~ctxt:oasis_ctxt txt)
                | None ->
-                   try 
-                     ignore (value.parse ~ctxt:!oasis_ctxt txt);
-                     assert_failure 
+                   try
+                     ignore (value.parse ~ctxt:oasis_ctxt txt);
+                     assert_failure
                        (Printf.sprintf
                           "License '%s' is not valid but parse without problem"
                           txt)
@@ -67,22 +71,28 @@ let tests =
                      ()))
 
        [
-         "BSD4", 
+         "BSD-4-clause",
          mk bsd4;
 
-         "BSD4-1.0+", 
-         mk ~v:(VersionOrLater (ver "1.0")) bsd4;
+         "BSD-4-clause-1.0+",
+         None;
+        
+         (* TODO: Test also BSD3 because it was a deprecated synonym BSD-3-clause, same for 
+            BSD2. *)
 
-         "BSD3", 
+         "BSD-3-clause",
          mk bsd3;
 
-         "GPL",  
+         "BSD3",
+         mk bsd3;
+
+         "GPL",
          mk gpl;
 
-         "GPL-2", 
+         "GPL-2",
          mk ~v:(Version (ver "2")) gpl;
 
-         "GPL-2+", 
+         "GPL-2+",
          mk ~v:(VersionOrLater (ver "2")) gpl;
 
          "LGPL-2.1 with OCaml linking exception",
@@ -118,7 +128,7 @@ let tests =
                   mk_base artistic;
                 ]));
 
-         "GPL-2+ and BSD3",
+         "GPL-2+ and BSD-3-clause",
          Some
            (DEP5License
              (DEP5And
@@ -127,7 +137,7 @@ let tests =
                   mk_base bsd3;
                 ]));
 
-         "GPL-2+ or Artistic-2.0, and BSD3",
+         "GPL-2+ or Artistic-2.0, and BSD-3-clause",
          Some
            (DEP5License
              (DEP5And
