@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,28 +20,35 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (** BaseLog tests
     @author Sylvain Le Gall
   *)
 
-open OUnit
+
+open OUnit2
 open TestCommon
 open BaseLog
 
+
 let tests =
   let test_of_vector (nm, f) =
-    nm >:: 
-    bracket
-      ignore
-      f
-      (fun () ->
-         FileUtil.rm [BaseLog.default_filename])
+    nm >::
+    (* TODO: depends on chdir! *)
+    (fun test_ctxt ->
+       let () = bracket
+                  ignore
+                  (fun () test_ctxt ->
+                     FileUtil.rm [BaseLog.default_filename])
+                  test_ctxt
+       in
+         f ())
   in
   let assert_equal_log msg exp =
     assert_equal
       ~msg
       ~printer:(fun lst ->
-                  String.concat ", " 
+                  String.concat ", "
                     (List.map
                        (fun (e, d) -> Printf.sprintf "%S %S" e d)
                        lst))
@@ -54,22 +62,22 @@ let tests =
          "normal",
          (fun () ->
             register "toto" "mytoto";
-            assert_bool 
-              "Event toto exists" 
+            assert_bool
+              "Event toto exists"
               (exists "toto" "mytoto");
             unregister "toto" "mytoto";
-            assert_bool 
-              "Event toto doesn't exist" 
+            assert_bool
+              "Event toto doesn't exist"
               (not (exists "toto" "mytoto")));
 
          "double",
          (fun () ->
             register "toto" "mytoto";
-            assert_equal_log 
+            assert_equal_log
               "Log contains 1 element"
               ["toto", "mytoto"];
             register "toto" "mytoto";
-            assert_equal_log 
+            assert_equal_log
               "Log still contains 1 element"
               ["toto", "mytoto"])
        ])
