@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,18 +20,21 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (** Tests for PropList
     @author Sylvain Le Gall
   *)
 
-open OUnit;;
-open TestCommon;;
-open PropList;;
-open PropList.Field;;
+
+open OUnit2
+open TestCommon
+open PropList
+open PropList.Field
+
 
 let tests =
   "PropList" >::
-  (fun () ->
+  (fun test_ctxt ->
      let toto =
        Field.create
          ~default:1
@@ -43,7 +47,7 @@ let tests =
      in
 
      let assert_equal_int =
-       assert_equal 
+       assert_equal
          ~printer:string_of_int
      in
 
@@ -65,4 +69,3 @@ let tests =
          (fget data toto);
 
        ())
-;;
