@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -24,25 +25,29 @@
     @author Sylvain Le Gall
   *)
 
+
 open TestCommon
-open OUnit
+open OUnit2
+
 
 let tests =
-  let test_of_vector (fn, qa) = 
-    let q, a = 
+  let test_of_vector (fn, qa) =
+    let q, a =
       List.split qa
     in
-    let test_name = 
-      Printf.sprintf 
-        "query('%s', %s)" 
+    let test_name =
+      Printf.sprintf
+        "query('%s', %s)"
         fn (String.concat ", " q)
     in
       test_name >::
-      (fun () ->
-         assert_oasis_cli 
+      (fun test_ctxt ->
+         assert_oasis_cli
+           ~ctxt:test_ctxt
            ~output:((String.concat "\n" a)^"\n")
            ~unorder:true
-           (["-quiet"; "query"; "-oasis"; in_data fn] @ q))
+           (["-quiet"; "query"; "-oasis";
+             in_testdata_dir test_ctxt ["TestQuery"; fn]] @ q))
   in
 
     "query" >:::
@@ -56,7 +61,7 @@ let tests =
          ["Flag(devmod).Default", "false"];
 
          "test10.oasis",
-         ["ListSections", 
+         ["ListSections",
           "Test(main)\nFlag(test)"];
 
          "test10.oasis",
