@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -24,94 +25,90 @@
     @author Sylvain Le Gall
   *)
 
+
 open TestCommon
 open Expect
 open ExpectPcre
-open OUnit
+open OUnit2
 open OASISTypes
 
-let () = 
-  METAPlugin.init ()
+
+let () = METAPlugin.init ()
+
 
 module MapString = Map.Make(String)
 
-let tests = 
 
-  let assert_exit_code_equal exp rel = 
-      assert_equal 
-        ~msg:"exit code"
-        ~printer:(function 
-                    | Unix.WEXITED i 
-                    | Unix.WSIGNALED i 
-                    | Unix.WSTOPPED i -> 
-                        string_of_int i)
-        exp
-        rel
+let tests =
+
+  let assert_exit_code_equal exp_list rel =
+    let string_of_exit_code =
+      function
+        | Unix.WEXITED i
+        | Unix.WSIGNALED i
+        | Unix.WSTOPPED i ->
+            string_of_int i
+    in
+    if not (List.mem rel exp_list) then
+      assert_failure
+        (Printf.sprintf
+           "Got exit code %s but expecting exit of to be in list [%s]."
+           (string_of_exit_code rel)
+           (String.concat ", " (List.map string_of_exit_code exp_list)))
   in
 
-  let with_quickstart_spawn args f exp_exit_code =
-    let args = 
-      ["quickstart"; "-machine"] @ args
-    in
-    let args = 
-      if not !dbug then 
-        "-quiet" :: args
-      else
-        "-debug" :: args
+  let with_quickstart_spawn test_ctxt args f exp_exit_codes =
+    let args =
+      (oasis_args test_ctxt) @ ["-debug"; "quickstart"; "-machine"] @ args
     in
 
-    let args = 
-      !oasis_args @ args
-    in
-
-    let () = 
-      if !dbug then
-        Printf.eprintf 
-          "Quickstart command line: %s\n%!" 
-          (String.concat " " (oasis () :: args))
-    in
-    let _, exit_code = 
+    let _, exit_code =
       try
+        logf test_ctxt `Info
+          "Quickstart command line: %s"
+          (String.concat " " (oasis_exec test_ctxt :: args));
         with_spawn
-          ~verbose:!dbug
-          ~timeout:(Some 0.1)
-          (oasis ())
+          ~verbose:true
+          ~verbose_output:(logf test_ctxt `Info "expect: %s")
+          ~timeout:None
+          (oasis_exec test_ctxt)
           (Array.of_list args)
-          (fun t () -> f t) 
+          (fun t () -> f t)
           ()
 
       with e ->
         Printexc.print_backtrace stderr;
         raise e
     in
-      assert_exit_code_equal 
-        exp_exit_code
+      assert_exit_code_equal
+        exp_exit_codes
         exit_code
   in
 
-  let run_quickstart args qa = 
+  let run_quickstart test_ctxt args qa =
     with_quickstart_spawn
+      test_ctxt
       args
       (fun t ->
-         let rec continue = 
+         let rec continue =
            function
              | [] ->
                  begin
                    ()
                  end
 
-             | qa -> 
+             | qa ->
                  begin
-                   let expectations = 
-                     let rec expectations' prev next = 
-                       match next with 
+                   let expectations =
+                     let rec expectations' prev next =
+                       match next with
                          | (q, a) :: tl ->
                              (* next QA if we chose this answer *)
-                             let qa' = 
+                             let qa' =
                                List.rev_append prev tl
                              in
-                             let q_pat = 
-                               match q with 
+                             let q_pat =
+                               match q with
                                  | "create_section" ->
                                      "create a(nother)? section\\? "
                                  | "end" ->
@@ -119,11 +116,11 @@ let tests =
                                  | "name" ->
                                      "'?name'?\\? "
                                  | q ->
-                                     Printf.sprintf 
+                                     Printf.sprintf
                                        "value for field '%s'\\? "
                                        q
                              in
-                             let q_rex = 
+                             let q_rex =
                                Pcre.regexp ~flags:[`CASELESS] q_pat
                              in
                                (`Rex q_rex, Some (a, qa'))
@@ -136,19 +133,19 @@ let tests =
                      in
                        expectations' [] qa
                    in
-                   let exp_q = 
+                   let exp_q =
                      expect t expectations None
                    in
 
-                     match exp_q with 
-                       | Some (a, qa) -> 
+                     match exp_q with
+                       | Some (a, qa) ->
                            send t (a^"\n");
                            continue qa
-                       | None -> 
-                           let assert_msg = 
+                       | None ->
+                           let assert_msg =
                              Printf.sprintf "expecting questions: %s"
                                (String.concat ", "
-                                  (List.map 
+                                  (List.map
                                      (fun (q, _) -> Printf.sprintf "%S" q)
                                      qa))
                            in
@@ -158,49 +155,36 @@ let tests =
            continue qa;
            assert_bool
              "wait for eof"
-             (expect t [`Eof, true] false)) 
-      (Unix.WEXITED 0)
+             (expect t [`Eof, true] false))
+      [Unix.WEXITED 0]
   in
 
-  let test_of_vector (nm, args, qa, post) = 
+  let test_of_vector (nm, args, qa, post) =
     nm >::
-    bracket 
-      (fun () ->
-         let pwd = FileUtil.pwd () in
-         let tmp = temp_dir () in
-           Sys.chdir tmp;
-           pwd, tmp)
-      (fun _ ->
-         run_quickstart args qa;
-         if !dbug then 
-           begin
-             let chn = open_in "_oasis" in
-             let () = 
-               try 
-                 while true do 
-                   prerr_endline (input_line chn)
-                 done
-               with End_of_file ->
-                 ()
-             in
-               close_in chn
-           end;
-         assert_oasis_cli ["check"];
+    (fun test_ctxt ->
+       let tmpdir = bracket_tmpdir test_ctxt in
+       let () =
+         with_bracket_chdir test_ctxt tmpdir
+           (fun test_ctxt ->
+              run_quickstart test_ctxt args qa)
+       in
+         dbug_file_content test_ctxt (Filename.concat tmpdir "_oasis");
+         (* TODO: rewrite with chdir *)
+         assert_oasis_cli ~ctxt:test_ctxt ["-C"; tmpdir; "check"];
          begin
-           try 
-             assert_oasis_cli ["setup"];
+           try
+             (* TODO: rewrite with chdir *)
+             assert_oasis_cli ~ctxt:test_ctxt ["-C"; tmpdir; "setup"]
            with e ->
              failwith "'oasis setup' failed but 'oasis check' succeed"
          end;
-         let pkg = 
-           OASISParse.from_file ~ctxt:!oasis_ctxt "_oasis"
+         let pkg =
+           OASISParse.from_file ~ctxt:oasis_ctxt
+             (Filename.concat tmpdir "_oasis")
          in
            post pkg)
-      (fun (pwd, tmp) ->
-         Sys.chdir pwd;
-         FileUtil.rm ~recurse:true [tmp])
   in
-  let test_simple_qa = 
+  let test_simple_qa =
     [
       "name",           "test";
       "version",        "0.0.1";
@@ -217,33 +201,33 @@ let tests =
     ]
   in
     "Quickstart" >:::
-    (List.map test_of_vector 
+    (List.map test_of_vector
        [
          "simple",
          [],
          test_simple_qa,
          (fun pkg ->
-            let () = 
-              assert_equal 
+            let () =
+              assert_equal
                 ~msg:"field name"
                 ~printer:(fun s -> s)
                 "test" pkg.name
             in
 
-            let sct = 
-              try 
-                OASISSection.section_find 
+            let sct =
+              try
+                OASISSection.section_find
                   (`Executable, "test")
                   pkg.sections
               with Not_found ->
                  failwith "Cannot find executable section 'test'"
             in
-              match sct with 
+              match sct with
                 | Executable (cs, bs, exec) ->
-                    assert_equal 
+                    assert_equal
                       ~msg:"mainis of test"
                       ~printer:(fun s -> s)
-                      "test.ml" exec.exec_main_is 
+                      "test.ml" exec.exec_main_is
                 | _ ->
                     assert false);
 
@@ -299,9 +283,11 @@ let tests =
            "xdevfilesmakefilenotargets", "";
            "xdevfilesenablemakefile", "";
            "xdevfilesenableconfigure", "";
+           "alphafeatures", "";
+           "betafeatures", "";
          ],
          (fun pkg ->
-            assert_equal 
+            assert_equal
               ~msg:"field name"
               ~printer:(fun s -> s)
               "test" pkg.name);
@@ -403,11 +389,15 @@ let tests =
            "xstdfilesinstallfilename", "bar";
            "xstdfilesauthors", "";
            "xstdfilesauthorsfilename", "baz";
+           "xocamlbuildplugintags", "";
+           "xocamlbuildextraargs", "";
+           "alphafeatures", "";
+           "betafeatures", "";
          ],
          (fun pkg ->
-            let libtest = 
-              let res = 
-                OASISSection.section_find 
+            let libtest =
+              let res =
+                OASISSection.section_find
                   (`Library, "libtest")
                   pkg.sections
               in
@@ -416,25 +406,25 @@ let tests =
             let meta =
               METAPlugin.generator libtest.cs_data
             in
-              assert_equal 
+              assert_equal
                 ~msg:"field name"
                 ~printer:(fun s -> s)
                 "test" pkg.name;
               assert_equal
                 ~msg:"field XMETAType"
-                ~printer:(function 
+                ~printer:(function
                             | METAPlugin.METASyntax -> "syntax"
                             | METAPlugin.METALibrary -> "library")
                 METAPlugin.METASyntax
                 meta.METAPlugin.meta_type;
-              assert_equal 
+              assert_equal
                 ~msg:"field XMETADescription"
                 ~printer:(function
                             | Some s -> s
                             | None -> "<none>")
                 (Some "this is a test")
                 meta.METAPlugin.description;
-              assert_equal 
+              assert_equal
                 ~msg:"field XMETARequires"
                 ~printer:(function
                             | Some lst -> String.concat ", " lst
@@ -446,8 +436,9 @@ let tests =
     @
     [
       "error" >::
-      (fun () ->
+      (fun test_ctxt ->
          with_quickstart_spawn
+           test_ctxt
            []
            (fun t ->
               let q = `Prefix ("???name "), true in
@@ -460,9 +451,6 @@ let tests =
                   ~msg:"Second question"
                   true
                   (expect t [q] false))
-           (if Sys.os_type = "Win32" then
-              (Unix.WEXITED 2)
-            else
-              (Unix.WSIGNALED ~-8)));
+           [Unix.WEXITED 2; Unix.WSIGNALED ~-8]);
     ]
 
