@@ -1,7 +1,8 @@
 (******************************************************************************)
 (* OASIS: architecture for building OCaml libraries and applications          *)
 (*                                                                            *)
-(* Copyright (C) 2008-2010, OCamlCore SARL                                    *)
+(* Copyright (C) 2011-2013, Sylvain Le Gall                                   *)
+(* Copyright (C) 2008-2011, OCamlCore SARL                                    *)
 (*                                                                            *)
 (* This library is free software; you can redistribute it and/or modify it    *)
 (* under the terms of the GNU Lesser General Public License as published by   *)
@@ -19,20 +20,23 @@
 (* Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA              *)
 (******************************************************************************)
 
+
 (** Test OASISValues defined fields
     @author Sylvain Le Gall
   *)
 
+
 open Printf
 open TestCommon
 open OASISValues
-open OUnit
+open OUnit2
+
 
 let tests_url =
   List.map
     (fun txt ->
-     TestCase (fun () ->
-               let _a = url.parse ~ctxt:!oasis_ctxt txt in
+     test_case (fun test_ctxt ->
+               let _a = url.parse ~ctxt:oasis_ctxt txt in
                ()))
     ["https://oasis.forge.ocamlcore.org";
      "svn://scm.ocamlcore.org/svnroot/cryptokit/tags";
@@ -41,13 +45,14 @@ let tests_url =
      "git+ssh://test.com";
     ]
 
+
 let tests_command_line_options =
   List.map
     (fun (s, r) ->
-     let title = sprintf "command_line_options: %S" s in
-     title >:: (fun () ->
-                let p = command_line_options.parse ~ctxt:!oasis_ctxt s in
-                assert_equal p r))
+     (sprintf "command_line_options: %S" s) >::
+      (fun test_ctxt ->
+         let p = command_line_options.parse ~ctxt:oasis_ctxt s in
+           assert_equal p r))
   ["", [];
    "a", ["a"];
    "a b", ["a"; "b"];
@@ -79,18 +84,20 @@ let tests_command_line_options =
    ["-DEXTERNAL_EXP10"; "-L/sw/lib"; "-framework vecLib"];
   ]
 
+
 let tests_posix_quoting =
   List.map
     (fun s ->
-     let title = sprintf "OASISUtils.POSIXShell: %S" s in
-     title >:: (fun () ->
-                let q = OASISUtils.POSIXShell.escape s in
-                let u =
-                  if q.[0] = '"' then
-                    let q' = String.sub q 1 (String.length q - 2) in
-                    OASISUtils.POSIXShell.unescape q'
-                  else q in
-                assert_equal s u))
+      (sprintf "OASISUtils.POSIXShell: %S" s) >::
+      (fun test_ctxt ->
+         let q = OASISUtils.POSIXShell.escape s in
+         let u =
+           if q.[0] = '"' then
+             let q' = String.sub q 1 (String.length q - 2) in
+               OASISUtils.POSIXShell.unescape q'
+           else q
+         in
+           assert_equal s u))
     [""; "a b"; "a'b"; "echo \'abcd"; "a\\b"]
 
 
